import urllib2
from bs4 import BeautifulSoup
import json

response = urllib2.urlopen('http://www.sanger.ac.uk/resources/downloads/bacteria/nctc/')
html = response.read()

soup=BeautifulSoup(html)
table = soup.find("table")

headings = [th.get_text() for th in table.find("tr").find_all("th")]

dataset={}
for row in table.find_all("tr")[1:]:
    #
    print row 
    row1=  [td.get_text() for td in row.find_all("td")]
    print row1
    metadata={}
    cellname=''
    for i, td in enumerate(row.find_all("td")):
        #print metadata
        link=td.find('a')
        # print i, td
        
        
        if i==1:
            cellname=td.get_text()
            print cellname
        
        if i==3:
            # print td
#             ERR_soup=BeautifulSoup(td)
            ERR_links=[]
            potential_links = td.findAll('a')
            # print potential_links
            for potential_link in td.findAll('a'):
                
                ERR_links.append((potential_link.text, potential_link['href']))
            metadata[headings[i]]=ERR_links
            continue
        if link != None:
            link=link.get('href')
        metadata[headings[i]]=(td.get_text(),link)
    
    list_of_files={}
    for run in metadata[headings[3]]:
        link_to_go=run[1]
        response1 = urllib2.urlopen(link_to_go+"&display=xml")
        xml = response1.read()
        xmlsoup = BeautifulSoup(xml)
        fllist=[]
        for data_block in xmlsoup.findAll('data_block'):
            for files in data_block.findAll('files'):
                for fle in files.findAll('file'):
                    fllist.append(fle['filename'])
        list_of_files[run[0]]=fllist
    metadata['file_paths']=list_of_files
#     print xml
    dataset[cellname]=metadata


with open('NCTC.json', 'w') as outfile:
    json.dump(dataset, outfile)




#!/usr/bin/env python

# coding: utf-8

# In[115]:

import networkx as nx
import random
import sys
import numpy as np
import ujson
from colormap import rgb2hex
import operator
import matplotlib.colors
# print G.number_of_edges(),G.number_of_nodes()


# In[3]:

def write_graph(G,flname):
    with open(flname,'w') as f:
        for edge in G.edges_iter():
            f.write(str(edge[0])+'\t'+str(edge[1])+'\n')


# In[4]:

def write_graph2(G,Ginfo,flname):

    count_no = 0
    count_yes = 0

    with open(flname,'w') as f:
        for edge in G.edges_iter():


            if (edge[0],edge[1]) not in Ginfo:
                count_no += 1
                print "not found"
                continue
            else:
                count_yes += 1


#             line = Ginfo[(edge[0],edge[1])]
#             line_sp = line.split(' ')

#             f.write(str(edge[0])+' '+str(edge[1]))
#             for j in range(2,len(line_sp)):
#                 f.write(' '+line_sp[j])

            f.write(Ginfo[(edge[0],edge[1])]+'\n')

    print count_no, count_yes







# In[7]:

def prune_graph(graph,in_hinges,out_hinges,reverse=False):

    H=nx.DiGraph()
    if reverse:
        G=nx.reverse(graph,copy=True)
    else:
        G=graph
    start_nodes = [x for x in G.nodes() if G.in_degree(x) ==0]

    in_hinges = list(in_hinges.intersection(set(G.nodes())))
    out_hinges = list(out_hinges.intersection(set(G.nodes())))

    if reverse:
        for node in in_hinges:
            for successor in G.successors(node):
#                 H.add_edge(node,successor)
                H.add_node(successor)
        for node in out_hinges:
            H.add_node(node)
    else:
        for node in out_hinges:
            for successor in G.successors(node):
#                 H.add_edge(node,successor)
                H.add_node(successor)
        for node in in_hinges:
            H.add_node(node)
    map(H.add_node,start_nodes)
    all_vertices=set(G.nodes())
    current_vertices=set(H.nodes())
    undiscovered_vertices=all_vertices-current_vertices
    last_discovered_vertices=current_vertices
    while undiscovered_vertices:
        discovered_vertices_set=set([x for node in last_discovered_vertices
                                  for x in G.successors(node)
                                  if x not in current_vertices])
        for vertex in discovered_vertices_set:
            for v_predecessor in G.predecessors(vertex):
                if v_predecessor in current_vertices:
                    H.add_edge(v_predecessor,vertex)
                    break
        current_vertices=current_vertices.union(discovered_vertices_set)
#         print len(undiscovered_vertices)
        if len(discovered_vertices_set)==0:
            print last_discovered_vertices
            print 'did not reach all nodes'
            print 'size of G: '+str(len(G.nodes()))
            print 'size of H: '+str(len(H.nodes()))
#             return H

            rand_node = list(undiscovered_vertices)[0]

            discovered_vertices_set.add(rand_node)


        last_discovered_vertices=discovered_vertices_set
        undiscovered_vertices=all_vertices-current_vertices
#     if reverse:
#         for vertex in out_hinges:
#             for v_predecessor in G.predecessors(vertex):
#                 H.add_edge(v_predecessor,vertex)
#     else:
#         for vertex in in_hinges:
#             for v_predecessor in G.predecessors(vertex):
#                 H.add_edge(v_predecessor,vertex)
    if reverse:
        for node in in_hinges:
            for successor in G.successors(node):
                H.add_edge(node,successor)
        for node in out_hinges:
            for predecessor in G.predecessors(node):
                H.add_edge(predecessor,node)
    else:
        for node in out_hinges:
            for successor in G.successors(node):
                H.add_edge(node,successor)
        for node in in_hinges:
            for predecessor in G.predecessors(node):
                H.add_edge(predecessor,node)
    if reverse:
        return nx.reverse(H)
    return H


# In[8]:

def dead_end_clipping(G,threshold):
#     H=nx.DiGraph()
    H = G.copy()
    start_nodes = set([x for x in H.nodes() if H.in_degree(x) ==0])

    for st_node in start_nodes:
        cur_path = [st_node]

        if len(H.successors(st_node)) == 1:
            cur_node = H.successors(st_node)[0]
            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1 and len(cur_path) < threshold + 2:
                cur_path.append(cur_node)
                cur_node = H.successors(cur_node)[0]


        if len(cur_path) <= threshold:
            for vertex in cur_path:
                H.remove_node(vertex)

    end_nodes = set([x for x in H.nodes() if H.out_degree(x) ==0])

    for end_node in end_nodes:
        cur_path = [end_node]
        if len(H.predecessors(end_node)) == 1:
            cur_node = H.predecessors(end_node)[0]
            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1 and len(cur_path) < threshold + 2:
                cur_path.append(cur_node)
                cur_node = H.predecessors(cur_node)[0]

        if len(cur_path) <= threshold:
            for vertex in cur_path:
                H.remove_node(vertex)

    return H



def rev_node(node):
    node_id = node.split('_')[0]

    return node_id + '_' + str(1-int(node.split('_')[1]))


def dead_end_clipping_sym(G,threshold,print_debug = False):
#     H=nx.DiGraph()
    H = G.copy()
    start_nodes = set([x for x in H.nodes() if H.in_degree(x) ==0])

    for st_node in start_nodes:

        if st_node not in H.nodes():
            continue

        cur_path = [st_node]

        cur_node = st_node
        if print_debug:
            print '----0'
            print st_node

        if len(H.successors(st_node)) == 1:
            cur_node = H.successors(st_node)[0]

            if print_debug:
                print '----1'

            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1 and len(cur_path) < threshold + 2:
                cur_path.append(cur_node)

                if print_debug:
                    print cur_node

                cur_node = H.successors(cur_node)[0]

                if len(cur_path) > threshold + 1:
                    break


        if print_debug:
            print '----2'
            print cur_path


        if len(cur_path) <= threshold and (H.in_degree(cur_node) > 1 or H.out_degree(cur_node) == 0):
            for vertex in cur_path:
                # try:
                if print_debug:
                    print 'about to delete ',vertex,rev_node(vertex)
                H.remove_node(vertex)
                H.remove_node(rev_node(vertex))
                # except:
                    # pass
                if print_debug:
                    print 'deleted ',vertex,rev_node(vertex)


    return H



# In[9]:


# This function is no longer used. See z_clipping_sym
def z_clipping(G,threshold,in_hinges,out_hinges,print_z = False):
    H = G.copy()

    start_nodes = set([x for x in H.nodes() if H.out_degree(x) > 1 and x not in out_hinges])

    for st_node in start_nodes:
        for sec_node in H.successors(st_node):

            if H.out_degree(st_node) == 1:
                break

            cur_node = sec_node
            cur_path = [[st_node,cur_node]]

            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:
                cur_path.append([cur_node,H.successors(cur_node)[0]])
                cur_node = H.successors(cur_node)[0]

                if len(cur_path) > threshold + 1:
                    break

            if len(cur_path) <= threshold and H.in_degree(cur_node) > 1 and H.out_degree(st_node) > 1 and cur_node not in in_hinges:
                if print_z:
                    print cur_path

                for edge in cur_path:
                    H.remove_edge(edge[0],edge[1])
                for j in range(len(cur_path)-1):
                    H.remove_node(cur_path[j][1])

    end_nodes = set([x for x in H.nodes() if H.in_degree(x) > 1 and x not in in_hinges])

    for end_node in end_nodes:
        for sec_node in H.predecessors(end_node):

            if H.in_degree(end_node) == 1:
                break


            cur_node = sec_node
            cur_path = [[cur_node,end_node]]

            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:
                cur_path.append([H.predecessors(cur_node)[0],cur_node])
                cur_node = H.predecessors(cur_node)[0]

                if len(cur_path) > threshold + 1:
                    break

            if len(cur_path) <= threshold and H.out_degree(cur_node) > 1 and H.in_degree(end_node) > 1 and cur_node not in out_hinges:
                if print_z:
                    print cur_path
                for edge in cur_path:
                    H.remove_edge(edge[0],edge[1])
                for j in range(len(cur_path)-1):
                    H.remove_node(cur_path[j][0])

    return H



def z_clipping_sym(G,threshold,in_hinges,out_hinges,print_z = False):

    H = G.copy()
    G0 = G.copy()

    start_nodes = set([x for x in H.nodes() if H.out_degree(x) > 1 and x not in out_hinges])

    for st_node in start_nodes:

        try:  # need this because we are deleting nodes inside loop
            H.successors(st_node)
        except:
            continue

        for sec_node in H.successors(st_node):

            if H.out_degree(st_node) == 1:
                break

            cur_node = sec_node
            cur_path = [[st_node,cur_node]]

            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:

                cur_path.append([cur_node,H.successors(cur_node)[0]])
                cur_node = H.successors(cur_node)[0]

                if len(cur_path) > threshold + 1:
                    break

            if len(cur_path) <= threshold and H.in_degree(cur_node) > 1 and H.out_degree(st_node) > 1 and cur_node not in in_hinges:
                if print_z:
                    print cur_path

                for edge in cur_path:

                    G0.edge[edge[0]][edge[1]]['z'] = 1
                    G0.edge[rev_node(edge[1])][rev_node(edge[0])]['z'] = 1

                    try:
                        H.remove_edge(edge[0],edge[1])
                        H.remove_edge(rev_node(edge[1]),rev_node(edge[0]))

                    except:
                        pass

                for j in range(len(cur_path)-1):

                    G0.node[cur_path[j][1]]['z'] = 1
                    G0.node[rev_node(cur_path[j][1])]['z'] = 1

                    try:
                        H.remove_node(cur_path[j][1])
                        H.remove_node(rev_node(cur_path[j][1]))

                    except:
                        pass


    return H, G0






# In[48]:

def merge_path(g,in_node,node,out_node):

    # g.add_edge(in_node,out_node,hinge_edge = -1,false_positive = 0)

    if g.edge[in_node][node]['intersection'] == 1 and g.edge[node][out_node]['intersection'] == 1:
        g.add_edge(in_node,out_node,hinge_edge = -1,intersection = 1,z=0)
    else:
        g.add_edge(in_node,out_node,hinge_edge = -1,intersection = 0,z=0)

    g.remove_node(node)



# In[121]:

def random_condensation(G,n_nodes,check_gt = False):

    g = G.copy()

    max_iter = 20000
    iter_cnt = 0

    while len(g.nodes()) > n_nodes and iter_cnt < max_iter:

        iter_cnt += 1

        node = g.nodes()[random.randrange(len(g.nodes()))]

        if g.in_degree(node) == 1 and g.out_degree(node) == 1:

            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    #print in_node, node, out_node
#                     merge_path(g,in_node,node,out_node)

                    bad_node=False
                    if check_gt:
                        for in_edge in g.in_edges(node):
                            if g.edge[in_edge[0]][in_edge[1]]['false_positive']==1:
                                bad_node=True
                        for out_edge in g.out_edges(node):
                            if g.edge[out_edge[0]][out_edge[1]]['false_positive']==1:
                                bad_node=True
                    if not bad_node:
                        #print in_node, node, out_node
                        merge_path(g,in_node,node,out_node)

    if iter_cnt >= max_iter:
        print "couldn't finish sparsification"+str(len(g.nodes()))

    return g



def random_condensation_sym(G,n_nodes,check_gt = False):

    g = G.copy()

    max_iter = 20000
    iter_cnt = 0

    while len(g.nodes()) > n_nodes and iter_cnt < max_iter:

        iter_cnt += 1

        node = g.nodes()[random.randrange(len(g.nodes()))]

        if g.in_degree(node) == 1 and g.out_degree(node) == 1:

            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    #print in_node, node, out_node
#                     merge_path(g,in_node,node,out_node)

                    bad_node=False
                    if check_gt:
                        for in_edge in g.in_edges(node):
                            if g.edge[in_edge[0]][in_edge[1]]['false_positive']==1:
                                bad_node=True
                        for out_edge in g.out_edges(node):
                            if g.edge[out_edge[0]][out_edge[1]]['false_positive']==1:
                                bad_node=True
                    if not bad_node:
                        #print in_node, node, out_node

                        try:
                            merge_path(g,in_node,node,out_node)
                            merge_path(g,rev_node(out_node),rev_node(node),rev_node(in_node))
                        except:
                            pass

    if iter_cnt >= max_iter:
        print "couldn't finish sparsification"+str(len(g.nodes()))

    return g


# In[118]:

def random_condensation2(g,n_nodes):

    g = G.copy()


    max_iter = 20000
    iter_cnt = 0

    while len(g.nodes()) > n_nodes and iter_cnt < max_iter:

        iter_cnt += 1

        node = g.nodes()[random.randrange(len(g.nodes()))]

        if g.in_degree(node) == 1 and g.out_degree(node) == 1:

            base_node=node.split("_")[0]
            orintation = node.split("_")[1]
            # if orintation=='1':
            #     node2=base_node+'_0'
            # else:
            #     node2=base_node+'_1'

            # print node,node2

            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]

            if g.node[node]['hinge']==0 and g.node[in_node]['hinge']==0  and g.node[out_node]['hinge']==0:
                if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                    if in_node != node and out_node != node and in_node != out_node:
                        bad_node=False
                        # print g.in_edges(node)
                        # print g.edge[g.in_edges(node)[0][0]][g.in_edges(node)[0][1]]
                        # print g.out_edges(node)


                        for in_edge in g.in_edges(node):
                            if g.edge[in_edge[0]][in_edge[1]]['false_positive']==1:
                                bad_node=True
                        for out_edge in g.out_edges(node):
                            if g.edge[out_edge[0]][out_edge[1]]['false_positive']==1:
                                bad_node=True
                        if not bad_node:
                            #print in_node, node, out_node
                            merge_path(g,in_node,node,out_node)



    if iter_cnt >= max_iter:
        print "couldn't finish sparsification: "+str(len(g.nodes()))


    return g




def bubble_bursting_sym(H,threshold,print_bubble = False):

    start_nodes = set([x for x in H.nodes() if H.out_degree(x) == 2])

    for st_node in start_nodes:

        try:  # need this because we are deleting nodes inside loop
            H.successors(st_node)[1]
        except:
            continue

        sec_node = H.successors(st_node)[0]

        cur_node = sec_node
        cur_path = [[st_node,cur_node]]

        while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:

            cur_path.append([cur_node,H.successors(cur_node)[0]])
            cur_node = H.successors(cur_node)[0]

            if len(cur_path) > threshold + 1:
                break

        end_node0 = cur_node
        cur_node = H.successors(st_node)[1]
        alt_path = [[st_node,cur_node]]

        while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:

            alt_path.append([cur_node,H.successors(cur_node)[0]])
            cur_node = H.successors(cur_node)[0]

            if len(alt_path) > threshold + 1:
                break


        if len(cur_path) <= threshold and len(alt_path) <= threshold and end_node0 == cur_node:

            if print_bubble:
                print 'found bubble'

            for edge in cur_path:

                # try:
                H.remove_edge(edge[0],edge[1])
                H.remove_edge(rev_node(edge[1]),rev_node(edge[0]))

                # except:
                #     pass

            for j in range(len(cur_path)-1):

                # try:
                H.remove_node(cur_path[j][1])
                H.remove_node(rev_node(cur_path[j][1]))

                # except:
                #     pass


    return H


def resolve_rep(g,rep_path,in_node,out_node):

    prefix = 'B'

    g.add_edge(in_node,prefix + rep_path[0],
        length=g.edge[in_node][rep_path[0]]['length'],
        read_a_match_start=g.edge[in_node][rep_path[0]]['read_a_match_start'],
        read_a_match_end=g.edge[in_node][rep_path[0]]['read_a_match_end'],
        read_b_match_start=g.edge[in_node][rep_path[0]]['read_b_match_start'],
        read_b_match_end=g.edge[in_node][rep_path[0]]['read_b_match_end'],
        read_a_match_start_raw=g.edge[in_node][rep_path[0]]['read_a_match_start_raw'],
        read_a_match_end_raw=g.edge[in_node][rep_path[0]]['read_a_match_end_raw'],
        read_b_match_start_raw=g.edge[in_node][rep_path[0]]['read_b_match_start_raw'],
        read_b_match_end_raw=g.edge[in_node][rep_path[0]]['read_b_match_end_raw'])
    g.remove_edge(in_node,rep_path[0])

    g.add_edge(prefix+rep_path[-1],out_node,
               length=g.edge[rep_path[-1]][out_node]['length'],
        read_a_match_start=g.edge[rep_path[-1]][out_node]['read_a_match_start'],
        read_a_match_end=g.edge[rep_path[-1]][out_node]['read_a_match_end'],
        read_b_match_start=g.edge[rep_path[-1]][out_node]['read_b_match_start'],
        read_b_match_end=g.edge[rep_path[-1]][out_node]['read_b_match_end'],
        read_a_match_start_raw=g.edge[rep_path[-1]][out_node]['read_a_match_start_raw'],
        read_a_match_end_raw=g.edge[rep_path[-1]][out_node]['read_a_match_end_raw'],
        read_b_match_start_raw=g.edge[rep_path[-1]][out_node]['read_b_match_start_raw'],
        read_b_match_end_raw=g.edge[rep_path[-1]][out_node]['read_b_match_end_raw'])
    g.remove_edge(rep_path[-1],out_node)


    g.add_edge(rev_node(prefix + rep_path[0]),rev_node(in_node),
               length =g.edge[rev_node(rep_path[0])][rev_node(in_node)]['length'],
        read_a_match_start=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_a_match_start'],
        read_a_match_end=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_a_match_end'],
        read_b_match_start=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_b_match_start'],
        read_b_match_end=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_b_match_end'],
        read_a_match_start_raw=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_a_match_start_raw'],
        read_a_match_end_raw=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_a_match_end_raw'],
        read_b_match_start_raw=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_b_match_start_raw'],
        read_b_match_end_raw=g.edge[rev_node(rep_path[0])][rev_node(in_node)]['read_b_match_end_raw'])
    g.remove_edge(rev_node(rep_path[0]),rev_node(in_node))
    g.add_edge(rev_node(out_node),rev_node(prefix+rep_path[-1]),
               length=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['length'],
        read_a_match_start=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_a_match_start'],
        read_a_match_end=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_a_match_end'],
        read_b_match_start=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_b_match_start'],
        read_b_match_end=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_b_match_end'],
        read_a_match_start_raw=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_a_match_start_raw'],
        read_a_match_end_raw=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_a_match_end_raw'],
        read_b_match_start_raw=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_b_match_start_raw'],
        read_b_match_end_raw=g.edge[rev_node(out_node)][rev_node(rep_path[-1])]['read_b_match_end_raw'])
    g.remove_edge(rev_node(out_node),rev_node(rep_path[-1]))




    for i in range(0,len(rep_path)-1):
        g.add_edge(prefix+rep_path[i],prefix+rep_path[i+1],
                   length=g.edge[rep_path[i]][rep_path[i+1]]['length'],
            read_a_match_start=g.edge[rep_path[i]][rep_path[i+1]]['read_a_match_start'],
            read_a_match_end=g.edge[rep_path[i]][rep_path[i+1]]['read_a_match_end'],
            read_b_match_start=g.edge[rep_path[i]][rep_path[i+1]]['read_b_match_start'],
            read_b_match_end=g.edge[rep_path[i]][rep_path[i+1]]['read_b_match_end'],
            read_a_match_start_raw=g.edge[rep_path[i]][rep_path[i+1]]['read_a_match_start_raw'],
            read_a_match_end_raw=g.edge[rep_path[i]][rep_path[i+1]]['read_a_match_end_raw'],
            read_b_match_start_raw=g.edge[rep_path[i]][rep_path[i+1]]['read_b_match_start_raw'],
            read_b_match_end_raw=g.edge[rep_path[i]][rep_path[i+1]]['read_b_match_end_raw'])
        g.add_edge(rev_node(prefix+rep_path[i+1]),rev_node(prefix+rep_path[i]),
                   length =g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['length'],
            read_a_match_start=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_a_match_start'],
            read_a_match_end=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_a_match_end'],
            read_b_match_start=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_b_match_start'],
            read_b_match_end=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_b_match_end'],
            read_a_match_start_raw=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_a_match_start_raw'],
            read_a_match_end_raw=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_a_match_end_raw'],
            read_b_match_start_raw=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_b_match_start_raw'],
            read_b_match_end_raw=g.edge[rev_node(rep_path[i+1])][rev_node(rep_path[i])]['read_b_match_end_raw'])




def loop_resolution(g,max_nodes,flank,print_debug = False):

    starting_nodes =  [x for x in g.nodes() if g.out_degree(x) == 2]

    if print_debug:
        print '----'
        print starting_nodes

    tandem = []

    for st_node in starting_nodes:


        if g.out_degree(st_node) != 2:
            continue

        if print_debug:
            print '----'
            print st_node


        for first_node in g.successors(st_node):


            if g.out_degree(st_node) != 2:
                continue

            if print_debug:
                print '----'
                print first_node

            other_successor = [x for x in g.successors(st_node) if x != first_node][0]

            next_node = first_node
            if print_debug:
                print 'going on loop'

            node_cnt = 0
            while g.in_degree(next_node) == 1 and g.out_degree(next_node) == 1 and node_cnt < max_nodes:
                node_cnt += 1
                in_node = next_node
                next_node = g.successors(next_node)[0]

            first_node_of_repeat = next_node

            if g.in_degree(next_node) == 2:
                prev_node = [x for x in g.predecessors(next_node) if x != in_node][0]
                node_cnt = 0
                while g.in_degree(prev_node) == 1 and g.out_degree(prev_node) == 1:
                    node_cnt += 1
                    prev_node = g.predecessors(prev_node)[0]
                    if node_cnt >= flank:
                        break
                if node_cnt < flank: # and prev_node != st_node:
                    continue


            next_node = other_successor
            node_cnt = 0
            while g.in_degree(next_node) == 1 and g.out_degree(next_node) == 1:
                node_cnt += 1
                next_node = g.successors(next_node)[0]
                if node_cnt >= flank:
                    break

            if node_cnt < flank: # and next_node != first_node_of_repeat:
                continue

            rep = [first_node_of_repeat]
            next_node = first_node_of_repeat

            node_cnt = 0

            if g.in_degree(next_node) == 2 and g.out_degree(next_node) == 1:
                next_double_node = g.successors(next_node)[0]
                rep.append(next_double_node)
            else:
                next_double_node = next_node

            while g.in_degree(next_double_node) == 1 and g.out_degree(next_double_node) == 1 and node_cnt < max_nodes:
                node_cnt += 1
                next_double_node = g.successors(next_double_node)[0]
                rep.append(next_double_node)


            if next_double_node == st_node:
                if print_debug:
                    print 'success!'
                    print 'rep is:'
                    print rep
                    print 'in_node and other_successor:'
                    print in_node, other_successor
                resolve_rep(g,rep,in_node,other_successor)
    #             print next_double_node

                if node_cnt < 5:

                    tandem.append(rep)



                continue

    if len(tandem) > 0:
        with open('tandem.txt', 'w') as tandemout:
            for rep in tandem:
                tandemout.write(str(rep))


    return g




# In[72]:


def add_groundtruth(g,json_file,in_hinges,out_hinges):

    mapping = ujson.load(json_file)

    print 'getting mapping'
    mapped_nodes=0
    print str(len(mapping))
    print str(len(g.nodes()))

    slack = 500
    max_chr = 0

    chr_length_dict = {}

    for node in g.nodes():
        # print node
        node_base=node.split("_")[0]
        # print node_base

        #print node
        g.node[node]['normpos'] = 0
        if mapping.has_key(node_base):
            g.node[node]['chr'] = mapping[node_base][0][2]+1
            g.node[node]['aln_start'] = min (mapping[node_base][0][0],mapping[node_base][0][1])
            g.node[node]['aln_end'] = max(mapping[node_base][0][1],mapping[node_base][0][0])


            # max_chr = max(g.node[node]['chr'],max_chr)
            # mapped_nodes+=1
        else:
            # pass
            g.node[node]['chr'] =  0
            g.node[node]['aln_start'] = 1
            g.node[node]['aln_end'] = 1
#             g.node[node]['aln_strand'] = 0

        if node in in_hinges or node in out_hinges:
            g.node[node]['hinge'] = 1
        else:
            g.node[node]['hinge'] = 0

        if g.node[node]['chr'] in chr_length_dict:
            chr_length_dict[g.node[node]['chr']] = max(g.node[node]['aln_end'], chr_length_dict[g.node[node]['chr']])
        else:
            chr_length_dict[g.node[node]['chr']] = max(g.node[node]['aln_end'], 1)

    chr_list = sorted(chr_length_dict.items(), key=operator.itemgetter(1), reverse=True)

    max_chr_len1 = max([g.node[x]['aln_end'] for x in  g.nodes()])
    max_chr_multiplier = 10**len(str(max_chr_len1))
    print [x for x in chr_list]
    chr_set =[x [0] for x in chr_list]
    print chr_set
    # red_bk = 102
    # green_bk = 102
    # blue_bk = 102
    colour_list = ['red', 'lawngreen', 'deepskyblue', 'deeppink', 'darkorange', 'purple', 'gold', 'mediumblue',   'saddlebrown', 'darkgreen']
    for colour in colour_list:
        print  matplotlib.colors.colorConverter.to_rgb(colour)
    for index, chrom in enumerate(chr_set):
        node_set = set([x for x in  g.nodes() if g.node[x]['chr'] == chrom])
        print chrom


        max_chr_len = max([g.node[x]['aln_end'] for x in  g.nodes() if g.node[x]['chr'] == chrom])
        # max_chr_multiplier = 10**len(str(max_chr_len))


        if index < 10:
            rgb_tuple = matplotlib.colors.colorConverter.to_rgb(colour_list[index])
            red = int(255*rgb_tuple[0])
            green = int(255*rgb_tuple[1])
            blue = int(255*rgb_tuple[2])
        else:
            red = random.randint(0,255)
            # green = random.randint(0,255)
            blue = random.randint(0,255)
            brightness = 200
            green  = max(0,min( 255,brightness - int((0.2126 *red +  0.0722 *blue)/0.7152 )))

        red_bk = max(red-100,0)
        blue_bk = max(blue-100,0)
        green_bk = max(green-100,0)

        print red,blue,green
        for node in node_set:
            g.node[node]['normpos'] = g.node[node]['chr'] * max_chr_multiplier + (g.node[node]['aln_end']/float(max_chr_len))*max_chr_multiplier
            lamda = (g.node[node]['aln_end']/max_chr_len)
            nd_red = (1-lamda)*red + lamda*red_bk
            nd_green = (1-lamda)*green + lamda*green_bk
            nd_blue = (1-lamda)*blue + lamda*blue_bk
            g.node[node]['color'] = rgb2hex(nd_red, nd_green, nd_blue)
            g.node[node]['color_r'] = nd_red
            g.node[node]['color_g'] = nd_green
            g.node[node]['color_b'] = nd_blue

    # max_chr_len = len(str(max_chr))

    # div_num = float(10**(max_chr_len))

    # for node in g.nodes():
    #     g.node[node]['normpos'] = (g.node[node]['chr'] + g.node[node]['aln_end']/float(chr_length_dict[g.node[node]['chr']]))/div_num

    for edge in g.edges_iter():
        in_node=edge[0]
        out_node=edge[1]

#         if ((g.node[in_node]['aln_start'] < g.node[out_node]['aln_start'] and
#             g.node[out_node]['aln_start'] < g.node[in_node]['aln_end']) or
#             (g.node[in_node]['aln_start'] < g.node[out_node]['aln_end'] and
#             g.node[out_node]['aln_end'] < g.node[in_node]['aln_end'])):
#             g.edge[in_node][out_node]['false_positive']=0
#         else:
#             g.edge[in_node][out_node]['false_positive']=1


        if ((g.node[in_node]['aln_start'] < g.node[out_node]['aln_start'] and
            g.node[out_node]['aln_start'] < g.node[in_node]['aln_end']) or
            (g.node[in_node]['aln_start'] < g.node[out_node]['aln_end'] and
            g.node[out_node]['aln_end'] < g.node[in_node]['aln_end'])):
            g.edge[in_node][out_node]['false_positive']=0
        else:
            g.edge[in_node][out_node]['false_positive']=1

    return g


def mark_skipped_edges(G,skipped_name):

    with open (skipped_name) as f:
        for lines in f:
            lines1=lines.split()

            if len(lines1) < 5:
                continue

            e1 = (lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4])

            if e1 in G.edges():
                G.edge[lines1[0] + "_" + lines1[3]][lines1[1] + "_" + lines1[4]]['skipped'] = 1
                G.edge[lines1[1] + "_" + str(1-int(lines1[4]))][lines1[0] + "_" + str(1-int(lines1[3]))]['skipped'] = 1





def add_annotation(g,in_hinges,out_hinges):

    for node in g.nodes():

        if node in in_hinges:
            g.node[node]['hinge'] = 1
        elif node in out_hinges:
            g.node[node]['hinge'] = -1
        else:
            g.node[node]['hinge'] = 0

    return g



def connect_strands(g):

    for node in g.nodes():
        revnode = rev_node(node)
        g.add_edge(node,revnode)
        g.add_edge(revnode,node)

    return g





def create_bidirected(g):

    h = nx.DiGraph()

    for u in g.nodes():

        for successor in g.successors(u):

            tail_id, tail_orientation = u.split('_')
            head_id, head_orientation = successor.split('_')

            h.add_edge(tail_id,head_id,tail_or = int(tail_orientation),head_or = int(head_orientation),
                read_a_match_start=g.edge[u][successor]['read_a_match_start'],
                read_a_match_end=g.edge[u][successor]['read_a_match_end'],
                read_b_match_start=g.edge[u][successor]['read_b_match_start'],
                read_b_match_end=g.edge[u][successor]['read_b_match_end'])


    st_nodes = [x for x in g if g.in_degree(x) != 1 or g.out_degree(x) > 1]

    for st_node in st_nodes:

        for sec_node in g.successors(st_node):

            cur_node = st_node
            cur_id = cur_node.split('_')[0]
            next_node = sec_node
            next_id = next_node.split('_')[0]

            if next_id in h.successors(cur_id) and cur_id in h.successors(next_id):
                h.remove_edge(next_id,cur_id)

            while g.in_degree(next_node) == 1 and g.out_degree(next_node) == 1:

                cur_node = next_node
                cur_id = cur_node.split('_')[0]
                next_node = g.successors(next_node)[0]
                next_id = next_node.split('_')[0]
                # else:
                #     print 'not in h'

                if next_id in h.successors(cur_id) and cur_id in h.successors(next_id):
                    h.remove_edge(next_id,cur_id)
                else:
                    break


    return h




def create_bidirected2(g):

    h = nx.DiGraph()

    for u in g.nodes():

        for successor in g.successors(u):

            tail_id, tail_orientation = u.split('_')
            head_id, head_orientation = successor.split('_')

            h.add_edge(tail_id,head_id)

            # h.add_edge(tail_id,head_id,tail_or = int(tail_orientation),head_or = int(head_orientation),
            #     read_a_match_start=g.edge[u][successor]['read_a_match_start'],
            #     read_a_match_end=g.edge[u][successor]['read_a_match_end'],
            #     read_b_match_start=g.edge[u][successor]['read_b_match_start'],
            #     read_b_match_end=g.edge[u][successor]['read_b_match_end'])


    st_nodes = [x for x in g if g.in_degree(x) != 1 or g.out_degree(x) > 1]

    for st_node in st_nodes:

        for sec_node in g.successors(st_node):

            cur_node = st_node
            cur_id = cur_node.split('_')[0]
            next_node = sec_node
            next_id = next_node.split('_')[0]

            if next_id in h.successors(cur_id) and cur_id in h.successors(next_id):
                h.remove_edge(next_id,cur_id)

            while g.in_degree(next_node) == 1 and g.out_degree(next_node) == 1:

                cur_node = next_node
                cur_id = cur_node.split('_')[0]
                next_node = g.successors(next_node)[0]
                next_id = next_node.split('_')[0]
                # else:
                #     print 'not in h'

                if next_id in h.successors(cur_id) and cur_id in h.successors(next_id):
                    h.remove_edge(next_id,cur_id)
                else:
                    break


    return g




def write_graphml(g,prefix,suffix,suffix1):
    h = g.copy()
    connect_strands(h)
    nx.write_graphml(h, prefix+suffix+'.'+'suffix1'+'.graphml')



flname = sys.argv[1]
# flname = '../pb_data/ecoli_shortened/ecoli4/ecolii2.edges.hinges'

prefix = flname.split('.')[0]

hingesname = sys.argv[2]
# hingesname = '../pb_data/ecoli_shortened/ecoli4/ecolii2.hinge.list'


suffix = sys.argv[3]

if len(sys.argv)==5:
    json_file = open(sys.argv[4])
else:
    json_file = None
# path = '../pb_data/ecoli_shortened/ecoli4/'
# suffix = 'i2'



# In[116]:

G = nx.DiGraph()

Ginfo = {}

with open (flname) as f:
    for lines in f:
        lines1=lines.split()

        if len(lines1) < 5:
            continue

        e1 = (lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4])
        # print lines1
        # e1_match1 = abs(int(lines1[6].lstrip('['))-int(lines1[7].rstrip(']')))
        # e1_match2 = abs(int(lines1[8].lstrip('['))-int(lines1[9].rstrip(']')))
        e1_match_len = int(lines1[2])
        ra_match_start = int(lines1[6].lstrip('['))
        ra_match_end = int(lines1[7].rstrip(']'))
        rb_match_start = int(lines1[8].lstrip('['))
        rb_match_end = int(lines1[9].rstrip(']'))

        ra_match_start_raw = int(lines1[-4].lstrip('['))
        ra_match_end_raw = int(lines1[-3].rstrip(']'))
        rb_match_start_raw = int(lines1[-2].lstrip('['))
        rb_match_end_raw = int(lines1[-1].rstrip(']'))


        if e1 in G.edges():
            G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],
                hinge_edge=int(lines1[5]),intersection=1,length=e1_match_len,z=0,
                read_a_match_start=ra_match_start,read_a_match_end=ra_match_end,
                read_b_match_start=rb_match_start,read_b_match_end=rb_match_end,
                read_a_match_start_raw=ra_match_start_raw,read_a_match_end_raw=ra_match_end_raw,
                read_b_match_start_raw=rb_match_start_raw,read_b_match_end_raw=rb_match_end_raw)
            G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),
                hinge_edge=int(lines1[5]),intersection=1,length=e1_match_len,z=0,
                read_a_match_start=rb_match_start,read_a_match_end=rb_match_end,
                read_b_match_start=ra_match_start,read_b_match_end=ra_match_end,
                read_a_match_start_raw=rb_match_start_raw,read_a_match_end_raw=rb_match_end_raw,
                read_b_match_start_raw=ra_match_start_raw,read_b_match_end_raw=ra_match_end_raw)
        else:
            G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],
                hinge_edge=int(lines1[5]),intersection=0,length=e1_match_len,z=0,
                read_a_match_start=ra_match_start,read_a_match_end=ra_match_end,
                read_b_match_start=rb_match_start,read_b_match_end=rb_match_end,
                read_a_match_start_raw=ra_match_start_raw,read_a_match_end_raw=ra_match_end_raw,
                read_b_match_start_raw=rb_match_start_raw,read_b_match_end_raw=rb_match_end_raw)
            G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),
                hinge_edge=int(lines1[5]),intersection=0,length=e1_match_len,z=0,
                read_a_match_start=rb_match_start,read_a_match_end=rb_match_end,
                read_b_match_start=ra_match_start,read_b_match_end=ra_match_end,
                read_a_match_start_raw=rb_match_start_raw,read_a_match_end_raw=rb_match_end_raw,
                read_b_match_start_raw=ra_match_start_raw,read_b_match_end_raw=ra_match_end_raw)



        towrite = lines1[0] + "_" + lines1[3] +' '+ lines1[1] + "_" + lines1[4] +' '+ lines1[2]+' '+str(int(lines1[11][:-1])-int(lines1[10][1:]))+' '+str(int(lines1[13][:-1])-int(lines1[12][1:]))
        Ginfo[(lines1[0] + "_" + lines1[3],lines1[1] + "_" + lines1[4])] = towrite

        towrite= lines1[1] + "_" + str(1-int(lines1[4])) +' '+ lines1[0] + "_" + str(1-int(lines1[3])) +' '+ lines1[2]+' '+str(int(lines1[13][:-1])-int(lines1[12][1:]))+' '+str(int(lines1[11][:-1])-int(lines1[10][1:]))
        Ginfo[(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])))] = towrite


nx.write_graphml(G, prefix+suffix+'.'+'G00'+'.graphml')



vertices=set()

in_hinges = set()
out_hinges = set()

with open (hingesname) as f:

    for lines in f:
        lines1=lines.split()

        if lines1[2] == '1':
            in_hinges.add(lines1[0]+'_0')
            out_hinges.add(lines1[0]+'_1')
        elif lines1[2] == '-1':
            in_hinges.add(lines1[0]+'_1')
            out_hinges.add(lines1[0]+'_0')




add_annotation(G,in_hinges,out_hinges)

# try:
mark_skipped_edges(G,flname.split('.')[0] + '.edges.skipped')
# except:
#     print "some error here"
#     pass



# json_file = open('../pb_data/ecoli_shortened/ecoli4/ecoli.mapping.1.json')


if json_file!= None:
    add_groundtruth(G,json_file,in_hinges,out_hinges)


# In[ ]:

G0 = G.copy()

# Actual pruning, clipping and z deletion occurs below


G0 = dead_end_clipping_sym(G0,10)

# G1=z_clipping_sym(G1,5,in_hinges,out_hinges)
G1,G0 = z_clipping_sym(G0,6,set(),set())
# G1=z_clipping_sym(G1,5,in_hinges,out_hinges)
# G1=z_clipping_sym(G1,5,in_hinges,out_hinges)
# G1=z_clipping_sym(G1,5,in_hinges,out_hinges)


G1 = bubble_bursting_sym(G1,20)

G1 = dead_end_clipping_sym(G1,20)

nx.write_graphml(G0, prefix+suffix+'.'+'G0'+'.graphml')
nx.write_graphml(G1, prefix+suffix+'.'+'G1'+'.graphml')


G2 = G1.copy()

Gs = random_condensation_sym(G1,1000)


loop_resolution(G2,500,50)

G2s = random_condensation_sym(G2,1000)





nx.write_graphml(G2, prefix+suffix+'.'+'G2'+'.graphml')

nx.write_graphml(Gs, prefix+suffix+'.'+'Gs'+'.graphml')

nx.write_graphml(G2s, prefix+suffix+'.'+'G2s'+'.graphml')

Gc = connect_strands(Gs)

nx.write_graphml(Gc, prefix+suffix+'.'+'Gc'+'.graphml')

G2c = connect_strands(G2s)

nx.write_graphml(G2c, prefix+suffix+'.'+'G2c'+'.graphml')

# G2b = create_bidirected2(G2)

# nx.write_graphml(G2b, prefix+suffix+'.'+'G2b'+'.graphml')




# H=prune_graph(G1,in_hinges,out_hinges)
# H=dead_end_clipping(H,5)


# I=prune_graph(H,in_hinges,out_hinges,True)
# I=dead_end_clipping(I,5)


# Gs = random_condensation(G1,2000)
# nx.write_graphml(Gs, path+'G'+suffix+'.graphml')
# write_graph(Gs,path+'G'+suffix+'.txt')

# Hs = random_condensation(H,2500)
# nx.write_graphml(Hs, path+'H'+suffix+'.graphml')
# write_graph(Hs,path+'H'+suffix+'.txt')

# Is = random_condensation(I,2500)
# nx.write_graphml(Is, path+'I'+suffix+'.graphml')
# write_graph(Is,path+'I'+suffix+'.txt')






#!/usr/bin/env python

import sys
import os
import subprocess
from parse_read import *

filename = sys.argv[1]
readarg = sys.argv[2]


stream = subprocess.Popen(["DBshow", filename ,readarg],
                                  stdout=subprocess.PIPE,bufsize=1)

reads = parse_read(stream.stdout) # generator

for read in reads:
    print read

#print result



#!/usr/bin/env python

import os
import argparse

ap = argparse.ArgumentParser(description="run LAsplit by splitting las into sizes of less than specified length")
ap.add_argument("las", help="path to las file to be split. assumed to be sorted.")
ap.add_argument("max_size", help="max size of any split file.", type=int, default=4, nargs='?')

args = ap.parse_args()

laspath = args.las
max_las_size = args.max_size


x = os.path.getsize(laspath)
num_divisions = (x/10**9)/max_las_size + 1
out_las_name = laspath.split('.las')[0]+'.# '

LAsplit_cmd = 'LAsplit -v '+out_las_name+ str(num_divisions) +' < ' + laspath
os.system(LAsplit_cmd)


#!/usr/bin/env python
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from ipywidgets.widgets import interact
import interface_utils as util
import sys

import os
os.environ['PATH'] += ':/data/pacbio_assembly/AwesomeAssembler/DALIGNER'
#print os.popen("export").read()

n = (sys.argv[1])
rst = []
with open(n) as f:
    for line in f:
        tmp = line.strip().split()
        t1 = tmp[0]
        if t1[-1] == '\'':
            t1 = t1[:-1]
        t2 = tmp[1]
        if t2[-1] == '\'':
            t2 = t2[:-1]
        rst.append((int(t1)+1, int(tmp[2])))
        #rst.append(int(t2)+1)

#rst = range(1,1399)
path = '/data/pacbio_assembly/AwesomeAssembler/data/'
aln = []
for i,e in enumerate(rst):
    n = e[0]
    print i,n
    li = list(util.get_alignments_mapping(path+'ecoli', path + 'ecoli.ref', path +'ecoli.ecoli.ref.las', [n]))
    if (len(li) > 0):
        item = sorted(li, key=lambda x:x[4] - x[3], reverse = True)[0]
        aln.append(item)

print aln[0:20]

#aln.sort(key = lambda x:x[2])

alns = []
current_b = aln[0][2]
aln_group = []

for item in aln:
    if current_b != item[2]:
        alns.append(aln_group)
        aln_group = []
        aln_group.append(item)
        current_b = item[2]
    else:
        aln_group.append(item)

num = len(alns)
print len(aln), len(alns)

#print [len(item) for item in alns]
#print [item[0:3] for item in aln]

#alns.sort(key = lambda x:min([item[3] for item in x]))

#size_chunk = num/grid_size
#for i in range(grid_size):
#    aln[i*size_chunk:min((i+1)*size_chunk, num)] = sorted(aln[i*size_chunk:min((i+1)*size_chunk, num)],key = lambda x: x[4]-x[3] ,reverse=True)

plt.figure(figsize = (15,10))
plt.axes()
#plt.gca().axes.get_yaxis().set_visible(False)
l = aln[0][5]
tip = l/5000
ed = l/2000
grid_size = 1.0
plt.xlim(-2000,l+2000)
plt.ylim(-5,num*grid_size)

points = [[0,0], [l,0], [l+tip,grid_size/4], [l,grid_size/2], [0,grid_size/2]]
#rectangle = plt.Rectangle((0, 0), l, 5, fc='r',ec = 'none')
polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.6)
plt.gca().add_patch(polygon)

dotted_line = plt.Line2D((0, 0), (0, num*grid_size ),ls='-.')
plt.gca().add_line(dotted_line)

dotted_line2 = plt.Line2D((l, l), (0, num*grid_size ),ls='-.')
plt.gca().add_line(dotted_line2)

for i,aln_group in enumerate(alns):
    for item in aln_group:
        abpos = item[3]
        aepos = item[4]
        bbpos = item[6]
        bepos = item[7]
        blen = item[8]
        strand = item[0]
        points_start = []
        points_end = []

        if strand == 'n':
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos + ed+ tip, (i+1)*grid_size + grid_size/4],  [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size]]
        else:
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2], [abpos - tip, (i+1)*grid_size + grid_size/4]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size],  [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size],[abpos-tip, (i+1)*grid_size+grid_size/4], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2],[abpos-ed-tip, (i+1)*grid_size+grid_size/4], [abpos-ed, (i+1)*grid_size]]

        polygon = plt.Polygon(points,fc = 'b', ec = 'none', alpha = 0.6)
        polygon.set_url("http://shannon.stanford.edu:5000/aln" + str(item[2]+1) + ".pdf")
        plt.gca().add_patch(polygon)

        if points_end != []:
            polygon2 = plt.Polygon(points_end,fc = 'g', ec = 'none', alpha = 0.6)
            plt.gca().add_patch(polygon2)

        if points_start != []:
            polygon2 = plt.Polygon(points_start,fc = 'g', ec = 'none', alpha = 0.6)
            plt.gca().add_patch(polygon2)

plt.savefig('mapping/map.' + str(n)+ '.svg')



#!/usr/bin/env python

import networkx as nx
import random
import sys
from collections import Counter
import json



# This script creates a graphml file from the hgraph file


def read_graph(filename,gt_file):

    with open(gt_file) as f:
        read_dict=json.load(f)
    g = nx.DiGraph()

    with open (filename) as f:
        for lines in f:
            lines1=lines.split()
            g.add_node(lines1[0] + "_" + lines1[2])
            g.add_node(lines1[1] + "_" + lines1[3])
            if lines1[0] in read_dict:
                g.node[lines1[0] + "_" + lines1[2]]['aln_start']=min(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
                g.node[lines1[0] + "_" + lines1[2]]['aln_end']=max(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
            else:
                g.node[lines1[0] + "_" + lines1[2]]['aln_start']=0
                g.node[lines1[0] + "_" + lines1[2]]['aln_end']=0
            if lines1[1] in read_dict:
                g.node[lines1[1] + "_" + lines1[3]]['aln_start']=min(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
                g.node[lines1[1] + "_" + lines1[3]]['aln_end']=max(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
            else:
                g.node[lines1[1] + "_" + lines1[3]]['aln_start']=0
                g.node[lines1[1] + "_" + lines1[3]]['aln_end']=0

            g.node[lines1[0] + "_" + lines1[2]]['active']=1
            g.node[lines1[1] + "_" + lines1[3]]['active']=int(lines1[4])
            g.add_edge(lines1[0] + "_" + lines1[2], lines1[1] + "_" + lines1[3])
    
    
    nx.write_graphml(g, filename.split('.')[0]+'_hgraph.graphml')
    
    print nx.number_weakly_connected_components(g)
    print nx.number_strongly_connected_components(g)
  

if __name__ == "__main__":   

    read_graph(sys.argv[1],sys.argv[2])








#!/usr/bin/env python
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from ipywidgets.widgets import interact
import interface_utils as util
import sys

import os
os.environ['PATH'] += ':~/AwesomeAssembler/DALIGNER'
#print os.popen("export").read()

left = int(sys.argv[1])
right = int(sys.argv[2])

ref = sys.argv[3]
read = sys.argv[4]
las = sys.argv[5]
contig = sys.argv[6]
length_th = int(sys.argv[7])
bridge_begin = int(sys.argv[8])
bridge_end = int(sys.argv[9])

#path = '/data/pacbio_assembly/AwesomeAssembler/data/ecoli/'
aln = []

#bb = []
#with open('ecoli.linear.edges') as f:
#    for line in f:
#        e = line.split(" ")[0]
#        if e[-1] == '\'':
#            e = e[:-1]
#
#        bb.append(int(e))
#
#print bb
#
#bb = set(bb)


for i,item in enumerate(util.get_alignments_mapping3(ref, read, las, contig)):
    #if i%2000 == 0:
    #    print i, item

    if item[3] >= left and item[4] <= right and item[4] - item[3] > length_th:
        aln.append(item)



covy = np.zeros((right - left, ))
for item in aln:
    covy[item[3] - left : item[4] - left] += 1

covx = np.arange(left, right)

#for i in range(0, len(covx), 10):
#    print covx[i], covy[i]

print 'number:',len(aln)
aln.sort(key = lambda x:x[2])

alns = []
current_b = aln[0][2]
aln_group = []

for item in aln:
    if current_b != item[2]:
        alns.append(aln_group)
        aln_group = []
        aln_group.append(item)
        current_b = item[2]
    else:
        aln_group.append(item)

num = len(alns)

print len(aln), len(alns)

alns.sort(key = lambda x:min([item[3] for item in x]))



fig = plt.figure(figsize = (15,10))
plt.axes()
ax1 = plt.subplot2grid((6,6), (0, 0), colspan=6, rowspan=4)
ax2 = plt.subplot2grid((6,6), (4, 0), colspan=6, rowspan=1, sharex = ax1)


#plt.gca().axes.get_yaxis().set_visible(False)
#l = aln[0][5]
tip = (right-left)/5000
ed = (right-left)/2000
grid_size = 1.0
ax1.set_xlim(left-2000,right+2000)
ax1.set_ylim(-5,num*grid_size)

points = [[left,0], [right,0], [right+tip,grid_size/4], [right,grid_size/2], [left,grid_size/2]]
#rectangle = plt.Rectangle((0, 0), l, 5, fc='r',ec = 'none')
polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.6)
ax1.add_patch(polygon)

dotted_line = plt.Line2D((left, left), (0, num*grid_size ),ls='-.')
ax1.add_line(dotted_line)

dotted_line2 = plt.Line2D((right, right), (0, num*grid_size ),ls='-.')
ax1.add_line(dotted_line2)

for i,aln_group in enumerate(alns):
    for item in aln_group:
        abpos = item[3]
        aepos = item[4]
        if abpos < bridge_begin+200 and aepos > bridge_end-200:
            print item
        bbpos = item[6]
        bepos = item[7]
        blen = item[8]
        strand = item[0]
        points_start = []
        points_end = []

        if strand == 'n':
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos + ed+ tip, (i+1)*grid_size + grid_size/4],  [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size]]
        else:
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2], [abpos - tip, (i+1)*grid_size + grid_size/4]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size],  [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size],[abpos-tip, (i+1)*grid_size+grid_size/4], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2],[abpos-ed-tip, (i+1)*grid_size+grid_size/4], [abpos-ed, (i+1)*grid_size]]

        #if item[2] in bb:
        #    polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.8)
        #else:
        polygon = plt.Polygon(points,fc = 'b', ec = 'none', alpha = 0.6)

        polygon.set_url("http://shannon.stanford.edu:5000/aln" + str(item[2]+1) + ".pdf")
        ax1.add_patch(polygon)

        if points_end != []:
            polygon2 = plt.Polygon(points_end,fc = 'g', ec = 'none', alpha = 0.6)
            ax1.add_patch(polygon2)

        if points_start != []:
            polygon2 = plt.Polygon(points_start,fc = 'g', ec = 'none', alpha = 0.6)
            ax1.add_patch(polygon2)


ax2.plot(covx, covy)
plt.xlabel('position')
ax1.set_ylabel('pile-o-gram')
ax2.set_ylabel('coverage')


plt.savefig('mapping/map.' + str(contig) + '_' + str(left) +'_'+ str(right)+ '.svg')



#!/usr/bin/env python

import sys
import os
from pbcore.io import FastaIO


def run(reader, writer):
    for i,record in enumerate(reader):
        if i%2 == 0:
            writer.writeRecord(record.header, record.sequence)


if __name__ == '__main__':
    iname, oname = sys.argv[1:3]
    reader = FastaIO.FastaReader(iname)
    writer = FastaIO.FastaWriter(oname)
    run(reader, writer)



#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter

def merge_simple_path(g):
    for node in g.nodes():
        #print g.in_degree(node), g.out_degree(node)
        if g.in_degree(node) == 1 and g.out_degree(node) == 1:
            
            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    #print in_node, node, out_node
                    merge_path(g,in_node,node,out_node)
                    
                              
def merge_two_nodes(g):
    for node in g.nodes():
        if g.in_degree(node) == 1 and g.out_degree(node) == 0:
            in_node = g.in_edges(node)[0][0]
            if g.out_degree(in_node) == 1:
                if in_node != node:
                    node_id = g.graph['aval']
                    g.graph['aval'] += 1
                    g.add_node(str(node_id), 
                        count = g.node[in_node]['count'] + g.node[node]['count'],
                        read = g.node[in_node]['read'] + '_' + g.node[node]['read'],
                        #aln_chr = g.node[node]['aln_chr']
                        )
                    g.remove_node(in_node)
                    g.remove_node(node)
            

def merge_path(g,in_node,node,out_node):
    #ov1 = find_overlap(g.node[in_node]['bases'], g.node[node]['bases'])
    #ov2 = find_overlap(g.node[node]['bases'], g.node[out_node]['bases'])
    
    node_id = g.graph['aval']
    g.graph['aval'] += 1
    #length = g.node[node]['length'] + g.node[in_node]['length'] + g.node[out_node]['length'] - ov1 - ov2
    #cov = (g.node[in_node]['cov'] * g.node[in_node]['length'] + g.node[node]['cov'] * g.node[node]['length']  + \
    #g.node[out_node]['cov'] * g.node[out_node]['length'])/float(length)
    #bases = g.node[in_node]['bases'][:-ov1] + g.node[node]['bases'] + g.node[out_node]['bases'][ov2:]
    
    g.add_node(str(node_id), 
        count = g.node[in_node]['count'] + g.node[node]['count'] + g.node[out_node]['count'],
        read = g.node[in_node]['read'] + '_' +  g.node[node]['read'] + '_' +g.node[out_node]['read'],
        #aln_chr = g.node[node]['aln_chr']
    )
    #g.add_node(str(node_id)+'-', bases = reverse_comp_bases(bases), length = length, cov = cov)
    
    for edge in g.in_edges(in_node):
        g.add_edge(edge[0],str(node_id))
    
    for edge in g.out_edges(out_node):
        g.add_edge(str(node_id),edge[1])
    
        
    g.remove_node(in_node)
    g.remove_node(node)
    g.remove_node(out_node)
    
def input1(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            if len(lines1) < 5:
                continue
            #print lines1
            g.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4], hinge_edge=int(lines1[5]))
            g.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))
    return g
            
def input2(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            g.add_edge(lines1[0], lines1[1])   
    return g

def run(filename, n_iter):
    
    
    f=open(filename)
    line1=f.readline()
    print line1
    f.close()
    if len(line1.split()) !=2:
	g=input1(filename)
    else:
	g=input2(filename)
    
    
    
    print nx.info(g)
    
    
    for node in g.nodes():
        g.node[node]['count'] = 1
        g.node[node]['read'] = node
        
        
    degree_sequence=sorted(g.degree().values(),reverse=True)
    print Counter(degree_sequence)
    for i in range(n_iter):
        for node in g.nodes():
            if g.in_degree(node) == 0:
                g.remove_node(node)
    
        print nx.info(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)
    
    
    g.graph['aval'] = 1000000000
    
    for i in range(5):
        merge_simple_path(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    try:
        import ujson
        mapping = ujson.load(open(filename.split('.')[0]+'.mapping.json'))
        
        print 'get mapping'
        
        for node in g.nodes():
            #print node
            if mapping.has_key(node):
                g.node[node]['aln_start'] = mapping[node][0]
                g.node[node]['aln_end'] = mapping[node][1]
                g.node[node]['aln_strand'] = mapping[node][2]
            else:
                g.node[node]['aln_start'] = 0
                g.node[node]['aln_end'] = 0
                g.node[node]['aln_strand'] = 0
                
    except:
        pass        
    
    nx.write_graphml(g, filename.split('.')[0]+'_condensed.graphml')
    
    print nx.number_weakly_connected_components(g)
    print nx.number_strongly_connected_components(g)
    
    
filename = sys.argv[1]
run(filename, 5)



#!/usr/bin/env python
import sys
import os
import subprocess
from parse_read import *
from parse_alignment import *

filename,filename2 = sys.argv[1:3]
alignmentname = sys.argv[3]
readarg = sys.argv[4]
k = int(sys.argv[5])


stream = subprocess.Popen(["LA4Awesome", filename, filename2 , alignmentname ,readarg],
                                  stdout=subprocess.PIPE, bufsize=1)

alignments = parse_alignment2(stream.stdout) # generator

d = {}
for alignment in alignments:
    if not d.has_key(alignment[2]):
        d[alignment[2]] = []
    d[alignment[2]].append([alignment[0],alignment[3],alignment[4], alignment[6], alignment[7], alignment[1]])
    
#print d

mapping = {}

for key,value in d.items():
    value.sort(key = lambda x:x[2]-x[1], reverse=True)
    alns = value[:k]
    max_val=alns[0][2]-alns[0][1]
    for aln in alns:
        if aln[2]-aln[1] > max_val/2.:
            if not mapping.has_key(str(key)):
                mapping[str(key)] = [(aln[1], aln[2],aln[-1], 1-int(aln[0] == 'n'))]
                # mapping[str(key)+'\''] = [(aln[2], aln[1],aln[-1], int(aln[0] == 'n'))]
            else:
                mapping[str(key)].append((aln[1], aln[2],aln[-1], 1-int(aln[0] == 'n')))
                # mapping[str(key)+'\''].append((aln[2], aln[1],aln[-1], int(aln[0] == 'n')))
            
#print mapping
import ujson
ujson.dump(mapping,open(filename2+'.mapping.'+str(k)+'.json','w'))



#!/usr/bin/env python

import sys

def parse_qv(stream = sys.stdin):    
    with stream as f:
        for l in f:
            if l[0] == 'I':
                yield(l.split()[-1])
            else:
                pass
                
#for qv in parse_qv():
#    print qv
# do whatever you want to do with the reads



#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter

def longest_path(G):
    dist = {} # stores [node, distance] pair
    for node in nx.topological_sort(G):
        # pairs of dist,node for all incoming edges
        pairs = [(dist[v][0]+1,v) for v in G.pred[node]] 
        if pairs:
            dist[node] = max(pairs)
        else:
            dist[node] = (0, node)
    node,(length,_)  = max(dist.items(), key=lambda x:x[1])
    path = []
    while length > 0:
        path.append(node)
        length,node = dist[node]
    return list(reversed(path))
    
    
filename = sys.argv[1]


g = nx.DiGraph()

with open(filename,'r') as f:
    for line in f.xreadlines():
        g.add_edge(*(line.strip().split('->')))


print nx.info(g)
degree_sequence=sorted(nx.degree(g).values(),reverse=True)
print Counter(degree_sequence)

for i in range(7):
    for node in g.nodes():
        if g.in_degree(node) == 0:
            g.remove_node(node)
            
    print nx.info(g)

degree_sequence=sorted(nx.degree(g).values(),reverse=True)
print Counter(degree_sequence)


def rev(string):
    if string[-1] == '\'':
        return string[:-1]
    else:
        return string+'\''

for edge in g.edges():
    g.add_edge(rev(edge[1]), rev(edge[0]))
    #print edge
    #print rev(edge[1]), rev(edge[0])

print nx.info(g)
nx.write_graphml(g, filename.split('.')[0]+'.graphml')
#print(list(nx.dfs_edges(g,sys.argv[2])))
#p=nx.shortest_path(g)



import sys
import os
import networkx as nx
from Bio import SeqIO



def comp_n50(contig_vec):
    if len(contig_vec) == 0:
        return 0
    sorted_lengths = sorted(contig_vec)

    total_length = sum(contig_vec)

    half_length = 0.5*total_length

    min_n50 = sorted_lengths[-1]
    max_n50 = 0

    for i in range(len(sorted_lengths)):
        sum_1 = sum(sorted_lengths[0:i+1])
        sum_2 = sum(sorted_lengths[i:])
        if sum_1 >= half_length and sum_2 >= half_length:
            min_n50 = min(sorted_lengths[i],min_n50)
            max_n50 = max(sorted_lengths[i],max_n50)

    return 0.5*(min_n50+max_n50)


hinging_n50 = -1
hinging_comp_n50 = -1
hgap_n50 = -1

count = 0
count1 = 0
count2 = 0


data_dict = {}




fullpath = '/data/pacbio_assembly/pb_data/NCTC/'

for nctc_name in os.listdir(fullpath):
	if 'NCTC' not in nctc_name:
		continue

	mypath = fullpath+nctc_name

	if not os.path.isdir(mypath):
		continue

	mypath = mypath+'/'

	count += 1

	hinging_n50 = -1
	hinging_comp_n50 = -1
	hgap_n50 = -1


	data_dict[nctc_name] = []

	draft_file = [x for x in os.listdir(mypath) if 'draft.graphml' in x]

	try:

		# flname = sys.argv[1]
		g = nx.read_graphml(mypath+draft_file[0])

		contig_lengths = []
		component_lengths = []


		for u in g.nodes():
		    contig_lengths.append(len(g.node[u]['segment']))
		    
		for c in nx.weakly_connected_components(g):
		    # we use set() so that we cannot double-count a two reverse complementary contigs
		    # in the same component
		    component_lengths.append(sum(set([len(g.node[u]['segment']) for u in c])))
		    
		component_lengths = set(component_lengths)

		hinging_n50 = comp_n50(contig_lengths)
		hinging_comp_n50 = comp_n50(component_lengths)

		count1+=1

	except:
		pass

	# print "contig n50: "+str(comp_n50(contig_lengths))
	# print "component n50: "+str(comp_n50(component_lengths))


	hgap_file = [x for x in os.listdir(mypath) if 'hgap.fasta' in x]


	try:

		hgap_file = hgap_file[0]
		hgap_contigs = [len(x) for x in SeqIO.parse(open(mypath+hgap_file),'fasta')]

		hgap_n50 = comp_n50(hgap_contigs)

		count2+=1

	except:
		pass


	with open(mypath+nctc_name+'.n50','w') as f:
		f.write('hinging'+'\t'+str(hinging_n50)+'\n')
		f.write('hinging_comp'+'\t'+str(hinging_comp_n50)+'\n')
		f.write('hgap'+'\t'+str(hgap_n50)+'\n')

	data_dict[nctc_name] = [hinging_n50,hinging_comp_n50,hgap_n50]

	print count
	print count1
	print count2



with open(fullpath+'computed.n50','w') as f:
	for nctc_name in data_dict:
		vec = data_dict[nctc_name]
		f.write(nctc_name+'\t'+str(vec[0])+'\t'+str(vec[1])+'\t'+str(vec[2])+'\n')






#!/usr/bin/env python

import sys
import os



def reverse_complement(bases):
    rev_comp={'A':'T','C':'G','G':'C','T':'A','N':'N'}
    return ''.join(map(lambda x :rev_comp[x],bases[::-1]))

def run(multifasta_path,intermediate_repeat_file_path, gt_file_path, gt_annotated_file_path):

	##Read chromosomes from multifasta file
	chrom={}
	cur_chrom=''
	start=True
	chr_num=0
	with open(multifasta_path,'r') as f:
	    for lines in f:
	        if lines[0]=='>':
	            
	            if start:
	                start=False
	                chr_num=int(lines.split()[0][1:])-1
	                print 'detect chr '+ str(chr_num)
	            else:
	                chrom[chr_num]=cur_chrom
	                print len(cur_chrom)
	                chr_num=int(lines.split()[0][1:])-1
	                print 'detect chr '+ str(chr_num)
	            cur_chrom=''
	        else:
	            cur_chrom+=lines.strip()
	print len(cur_chrom)            
	chrom[chr_num]=cur_chrom

	##Run mummer to get repeats
	mummer_cmd='mummer -maxmatch -b -c -l 1000 -L '+multifasta_path+' '+multifasta_path +' > '+intermediate_repeat_file_path 
	os.system(mummer_cmd)


	#Put repeats discovered by mummer in right form
	chr_num=0
	chr_repeats={}
	rev_com=False
	with open (intermediate_repeat_file_path) as f:
	    for line in f:
	        if line[0]=='>':
	            line1=line.strip().split()
	            chr_num=int(line1[1])-1
	            #print len(line1)
	            if len(line1)==6:
	                rev_com=True
	                chr_len=int(line1[5])
	            else:
	                rev_com=False
	                chr_len=int(line1[4])
	        else:
	            line1=line.strip().split()
	            chr2_num=int(line1[0])-1
	            chr2_start=int(line1[1])-1
	            chr1_start=int(line1[2])-1
	            rep_len=int(line1[3])
	            if not rev_com:
	                if not (chrom[chr_num][chr1_start:chr1_start+rep_len]
	                        ==chrom[chr2_num][chr2_start:chr2_start+rep_len]):
	                    print chr_num+1,line
	                if chr1_start==0 and rep_len==chr_len:
	                    continue
	                chr_repeats.setdefault(chr_num,[]).append((chr1_start,chr1_start+rep_len))
	            else:
	                if not (chrom[chr_num][chr1_start-rep_len+1:chr1_start+1]
	                        == reverse_complement(chrom[chr2_num][chr2_start:chr2_start+rep_len])):
	                    print chr_num+1,line,rev_com
	                chr_repeats.setdefault(chr_num,[]).append((chr1_start-rep_len+1,chr1_start+1))

	#Go through gt file and annotate reads that intersect with repeats.
	with open(gt_file_path) as f:
	    with open(gt_annotated_file_path,'w') as g:
	        for line in f:
	            line1=line.split()
	            cr=int(line1[1])
	            rd_st=int(line1[2])
	            rd_end=int(line1[3])
	            is_repeat=0
	            for tup in chr_repeats[cr]:
	                if ((rd_st >= tup[0] and rd_st <= tup[1]) or
	                    (rd_end >= tup[0] and rd_end <= tup[1])):
	                    is_repeat=1
	            line2=line.strip()+"\t"+str(is_repeat)+"\n"
	            g.write(line2)

if __name__ == '__main__':
	multifasta_path=sys.argv[1]
	gt_file_path=sys.argv[2]
	gt_annotated_file_path=sys.argv[3]
	intermediate_repeat_file_path='./repeats_discovered.txt'
	if len(sys.argv) > 4:
		intermediate_repeat_file_path=sys.argv[4]
	run(multifasta_path,intermediate_repeat_file_path, gt_file_path, gt_annotated_file_path)ß



#!/usr/bin/env python
import sys
import os
import subprocess
from parse_read import *
from parse_alignment import *

filename,filename2 = sys.argv[1:3]
alignmentname = sys.argv[3]
readarg = sys.argv[4]


stream = subprocess.Popen(["LA4Awesome", filename, filename2 , alignmentname ,readarg, '-F'],
                                  stdout=subprocess.PIPE, bufsize=1)

alignments = parse_alignment2(stream.stdout) # generator

d = {}
for alignment in alignments:
    if not d.has_key(alignment[2]):
        d[alignment[2]] = []
    d[alignment[2]].append([alignment[0],alignment[3],alignment[4], alignment[6], alignment[7], alignment[1]])
    
#print d

mapping = {}

for key,value in d.items():
    value.sort(key = lambda x:x[2]-x[1], reverse=True)
    aln = value[0]
    
    if aln[0] == 'n':
        mapping[str(key)] = (aln[1], aln[2],aln[-1], 0)
        mapping[str(key)+'\''] = (aln[2], aln[1],aln[-1], 1)
    else:
        mapping[str(key)] = (aln[2], aln[1], aln[-1], 1)
        mapping[str(key)+'\''] = (aln[1], aln[2], aln[-1], 0)
    
#print mapping
import ujson
ujson.dump(mapping,open(filename+'.mapping.json','w'))



#!/usr/bin/env python

#usage python get_single_strand.py <in-fasta> <out-fasta>

from pbcore.io import FastaIO
import sys

flpath = sys.argv[1]
outpath = sys.argv[2]
writer = FastaIO.FastaWriter(outpath)
reader = FastaIO.FastaReader(flpath)
j = 0
for i,record in enumerate(reader):
    if j%2 == 0:
        writer.writeRecord('Consensus'+str(j), record.sequence)
        j+=1


#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter



def merge_simple_path(g):
    for node in g.nodes():
        #print g.in_degree(node), g.out_degree(node)
        if g.in_degree(node) == 1 and g.out_degree(node) == 1:
            
            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    merge_path(g,in_node,node,out_node)
                    
                              
def merge_two_nodes(g):
    for node in g.nodes():
        if g.in_degree(node) == 1 and g.out_degree(node) == 0:
            in_node = g.in_edges(node)[0][0]
            if g.out_degree(in_node) == 1:
                if in_node != node:
                    node_id = g.graph['aval']
                    g.graph['aval'] += 1
                    g.add_node(str(node_id), 
                        count = g.node[in_node]['count'] + g.node[node]['count'],
                        read = g.node[in_node]['read'] + ':' + g.node[node]['read'],
                        #aln_chr = g.node[node]['aln_chr']
                        )
                    g.remove_node(in_node)
                    g.remove_node(node)
            

def merge_path(g,in_node,node,out_node):
    #ov1 = find_overlap(g.node[in_node]['bases'], g.node[node]['bases'])
    #ov2 = find_overlap(g.node[node]['bases'], g.node[out_node]['bases'])
    
    node_id = g.graph['aval']
    g.graph['aval'] += 1
    #length = g.node[node]['length'] + g.node[in_node]['length'] + g.node[out_node]['length'] - ov1 - ov2
    #cov = (g.node[in_node]['cov'] * g.node[in_node]['length'] + g.node[node]['cov'] * g.node[node]['length']  + \
    #g.node[out_node]['cov'] * g.node[out_node]['length'])/float(length)
    #bases = g.node[in_node]['bases'][:-ov1] + g.node[node]['bases'] + g.node[out_node]['bases'][ov2:]
    
    g.add_node(str(node_id), 
        count = g.node[in_node]['count'] + g.node[node]['count'] + g.node[out_node]['count'],
        read = g.node[in_node]['read'] + ':' +  g.node[node]['read'] + ':' +g.node[out_node]['read'],
        #aln_chr = g.node[node]['aln_chr']
    )
    #g.add_node(str(node_id)+'-', bases = reverse_comp_bases(bases), length = length, cov = cov)
    #print g.node[str(node_id)]['chr']

    for edge in g.in_edges(in_node):
        g.add_edge(edge[0],str(node_id),st_pc=g.edge[edge[0]][edge[1]]['st_pc'],end_pc=g.edge[edge[0]][edge[1]]['end_pc'])

    
    for edge in g.out_edges(out_node):
        g.add_edge(str(node_id),edge[1],st_pc=g.edge[edge[0]][edge[1]]['st_pc'],end_pc=g.edge[edge[0]][edge[1]]['end_pc'])
    
        
    g.remove_node(in_node)
    g.remove_node(node)
    g.remove_node(out_node)
    
def input1(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            if len(lines1) < 5:
                continue
            #print lines1
            g.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4], hinge_edge=int(lines1[5]))
            g.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))
    return g
            
def input2(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            g.add_edge(lines1[0], lines1[1])   
    return g

def run(filename, gt_file, n_iter):
    
    
    f=open(filename)
    line1=f.readline()
    print line1
    f.close()
    if len(line1.split()) !=2:
	   g=input1(filename)
    else:
	   g=input2(filename)
    
    read_to_chr_map={} 
    pos_dict = {}
    mapping_dict = {} 

    chr_lengths = {}
    for chr in range(14):
        chr_lengths[chr] = 1000

    with open(gt_file,'r') as f:
        for num, line in enumerate(f.readlines()):
            m = map(int, line.strip().split())
            # mapping_dict[num] = [min(m), max(m), int(m[0]>m[1])]
            read_to_chr_map[m[0]]= str(m[1])
            mapping_dict[num] = m[1]
            pos_dict[num] = [min(m[2],m[3]),max(m[2],m[3])]    
            # pos_dict[num] = [m[2],m[3],int(m[2]>m[3])]
            chr_lengths[m[1]] = max(chr_lengths[m[1]],max(m[2],m[3]))


    print nx.info(g)
    
    print "Chromosome lenghts:"
    print chr_lengths

    margin = 10000

    del_count = 0


    #print nx.info(g)
    print "Num reads read : "+str(len(read_to_chr_map))

    for cur_edge in g.edges():
        node0=int(cur_edge[0].split('_')[0])
        node1=int(cur_edge[1].split('_')[0])
        # g.edge[cur_edge[0]][cur_edge[1]]['st_pc'] = "{0:.2f}".format(1.0*pos_dict[node0][1]/chr_lengths[mapping_dict[node0]])
        # g.edge[cur_edge[0]][cur_edge[1]]['end_pc'] = "{0:.2f}".format(1.0*pos_dict[node1][0]/chr_lengths[mapping_dict[node1]])
        
        # st_pc is the "start percentage"; i.e., the percent location of edge[0] on its original chromosome
        # end_pc is the "end percentage"; i.e., the percent location of edge[1] on its original chromosome

        g.edge[cur_edge[0]][cur_edge[1]]['st_pc'] = 1.0*pos_dict[node0][1]/chr_lengths[mapping_dict[node0]]
        g.edge[cur_edge[0]][cur_edge[1]]['end_pc'] = 1.0*pos_dict[node1][0]/chr_lengths[mapping_dict[node1]]

    
    for node in g.nodes():
        nodeid=int(node.split('_')[0])

        if pos_dict[nodeid][0] < margin:
            g.remove_node(node)
            del_count += 1
            continue

        if pos_dict[nodeid][1] > chr_lengths[mapping_dict[nodeid]] - margin:
            g.remove_node(node)
            del_count += 1
            continue

        g.node[node]['count'] = 1
        g.node[node]['read'] = node
        #print str(nodeid), node,g.node[node]['chr']

    print "Deleted nodes: "+str(del_count)
        
        
    degree_sequence=sorted(g.degree().values(),reverse=True)
    print Counter(degree_sequence)
    for i in range(n_iter):
        for node in g.nodes():
            if g.in_degree(node) == 0:
                g.remove_node(node)
    
        print nx.info(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)
    
    
    g.graph['aval'] = 1000000000
    
    for i in range(5):
        merge_simple_path(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    h=nx.DiGraph()
    h.add_nodes_from(g)
    h.add_edges_from(g.edges())

    for cur_edge in h.edges():
        h.edge[cur_edge[0]][cur_edge[1]]['st_pc'] = g.edge[cur_edge[0]][cur_edge[1]]['st_pc']
        h.edge[cur_edge[0]][cur_edge[1]]['end_pc'] = g.edge[cur_edge[0]][cur_edge[1]]['end_pc']

    # h = g.copy()

    for node in g.nodes():
        reads_in_node=[int(x.split('_')[0]) for x in g.node[node]['read'].split(':')]
        try:
            chr_in_node=map(lambda x: read_to_chr_map[x], reads_in_node)
        except:
            print reads_in_node,g.node[node]['read']
            return
        chr_in_node_set=set(chr_in_node)
        if len(chr_in_node_set) ==1:
            h.node[node]['chr']=chr_in_node[0]
        else:
            h.node[node]['chr']= ':'.join(chr_in_node)

        h.node[node]['count']=g.node[node]['count']
        try:
            h.node[node]['read']=g.node[node]['read']
        except:
            pass


    try:
        import ujson
        mapping = ujson.load(open(filename.split('.')[0]+'.mapping.json'))
        
        print 'get mapping'
        
        for node in h.nodes():
            #print node
            if mapping.has_key(node):
                h.node[node]['aln_start'] = mapping[node][0]
                h.node[node]['aln_end'] = mapping[node][1]
                h.node[node]['aln_strand'] = mapping[node][2]
            else:
                h.node[node]['aln_start'] = 0
                h.node[node]['aln_end'] = 0
                h.node[node]['aln_strand'] = 0
                
    except:
        pass        


    
    nx.write_graphml(h, filename.split('.')[0]+'_condensed_annotated.graphml')
    nx.write_graphml(g, filename.split('.')[0]+'_G_condensed_annotated.graphml')
    
    print nx.number_weakly_connected_components(h)
    print nx.number_strongly_connected_components(h)
    
#

filename = sys.argv[1]
gt_file=sys.argv[2]
run(filename, gt_file,5)



#!/usr/bin/env python
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
#from ipywidgets.widgets import interact 
import interface_utils as util
import sys
import os
import linereader
os.environ['PATH'] += ':/data/pacbio_assembly/AwesomeAssembler/DALIGNER'
#print os.popen("export").read()

Qvd = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
    'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
    'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D',
    'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
    'Y']
Qvv = range(len(Qvd))[::-1]

QVdict = dict(zip(Qvd,Qvv))


dbname = sys.argv[1]
lasname = sys.argv[2]
n = int(sys.argv[3])
path =  os.getcwd()+'/'
coveragename = path + dbname + '.coverage.txt'

aln = []

coveragefile = linereader.copen(coveragename)

coverage = coveragefile.getline(n)
cov = coverage.split()[2:]
covx = []
covy = []
for item in cov:
    data = item.split(',')
    covx.append(int(data[0]))
    covy.append(int(data[1]))

qv = list(util.get_QV(path+dbname, [n]))[0]
qx = []
qy = []
ts = int(sys.argv[5])

if len(sys.argv) < 7:
    rev = 0
else:
    rev = int(sys.argv[6])

print 'rev', rev

for i in range(len(qv)):
    qx.append(i*ts)
    qy.append(QVdict[qv[i]])

for item in util.get_alignments2(path+dbname,path+lasname,[n]):
    aln.append(item)
    
if (len(aln) == 0):
    sys.exit()
#print aln[0:5]

aln.sort(key = lambda x:x[2])

alns = []
current_b = aln[0][2]
aln_group = []

for item in aln:
    if current_b != item[2]:
        alns.append(aln_group)
        aln_group = []
        aln_group.append(item)
        current_b = item[2]
    else:
        aln_group.append(item)
    
num = len(alns)
print len(aln), len(alns)

#print [len(item) for item in alns]
#print [item[0:3] for item in aln]

alns.sort(key = lambda x:min([item[3] for item in x]))

#size_chunk = num/grid_size
#for i in range(grid_size):
#    aln[i*size_chunk:min((i+1)*size_chunk, num)] = sorted(aln[i*size_chunk:min((i+1)*size_chunk, num)],key = lambda x: x[4]-x[3] ,reverse=True)

fig = plt.figure(figsize = (15,10))
plt.axes()
ax1 = plt.subplot2grid((6,6), (0, 0), colspan=6, rowspan=4)
ax2 = plt.subplot2grid((6,6), (4, 0), colspan=6, rowspan=1, sharex = ax1)
ax3 = plt.subplot2grid((6,6), (5, 0), colspan=6, rowspan=1, sharex = ax1)

#plt.gca().axes.get_yaxis().set_visible(False)
l = aln[0][5]
tip = l/200
ed = l/50
grid_size = 1.0
ax1.set_xlim(-2000,l+2000)
ax1.set_ylim(-5,num*grid_size)

if rev == 0:
    points = [[0,0], [l,0], [l+tip,grid_size/4], [l,grid_size/2], [0,grid_size/2]]
else:
    points = [[0,0], [-tip,grid_size/4], [0,grid_size/2],  [l,grid_size/2], [l,0]]
    


#rectangle = plt.Rectangle((0, 0), l, 5, fc='r',ec = 'none')
polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.6)
ax1.add_patch(polygon)

dotted_line = plt.Line2D((0, 0), (0, num*grid_size ),ls='-.')               
ax1.add_line(dotted_line)

dotted_line2 = plt.Line2D((l, l), (0, num*grid_size ),ls='-.')               
ax1.add_line(dotted_line2)

for i,aln_group in enumerate(alns):
    for item in aln_group:
        if rev == 0:
            abpos = item[3]
            aepos = item[4]
            bbpos = item[6]
            bepos = item[7]
            blen = item[8]
            strand = item[0]
        else:
            aepos = l - item[3]
            abpos = l - item[4]
            blen = item[8]
            bbpos = blen - item[7]
            bepos = blen - item[6]
            strand = item[0]
            if strand == 'n':
                strand = 'c'
            else:
                strand = 'n'
            
                    
        points_start = []
        points_end = []

        if strand == 'n':
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos + ed+ tip, (i+1)*grid_size + grid_size/4],  [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size]]
        else:
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2], [abpos - tip, (i+1)*grid_size + grid_size/4]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size],  [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size],[abpos-tip, (i+1)*grid_size+grid_size/4], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2],[abpos-ed-tip, (i+1)*grid_size+grid_size/4], [abpos-ed, (i+1)*grid_size]]

        polygon = plt.Polygon(points,fc = 'b', ec = 'none', alpha = 0.6)
        polygon.set_url('aln_svg' + str(item[2])+'.svg')
        ax1.add_patch(polygon)

        if points_end != []:
            polygon2 = plt.Polygon(points_end,fc = 'g', ec = 'none', alpha = 0.6)
            ax1.add_patch(polygon2)

        if points_start != []:
            polygon2 = plt.Polygon(points_start,fc = 'g', ec = 'none', alpha = 0.6)
            ax1.add_patch(polygon2)




if rev == 1:
    covx = [l -item for item in covx]
    qx = [l - item for item in qx]
    
ax2.plot(covx, covy)
ax3.plot(qx, qy)

plt.xlabel('position')
ax1.set_ylabel('pile-o-gram')
ax2.set_ylabel('coverage')
ax3.set_ylabel('i-qv')


plt.savefig(path + sys.argv[4] + '/aln_svg' + str(n) + '_' + str(rev)+ '.svg')



#!/usr/bin/env python

import sys
import os
import subprocess
from parse_read import *
from parse_alignment import *
from parse_qv import *

#filename = sys.argv[1]
#readarg = sys.argv[2]

def get_reads(filename, readlist):
    stream = subprocess.Popen(["DBshow", filename] + map(str,readlist),
                                      stdout=subprocess.PIPE,bufsize=1)
    reads = parse_read(stream.stdout) # generator
    return reads

def get_QV(filename, readlist):
    stream = subprocess.Popen(["DBdump", filename, '-i'] + map(str,readlist),
                                      stdout=subprocess.PIPE,bufsize=1)
    qv = parse_qv(stream.stdout) # generator
    return qv


def get_alignments(filename, readlist):
    stream = subprocess.Popen(["LAshow", filename,filename]+ map(str,readlist),
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment(stream.stdout) # generator
    return alignments


def get_alignments2(filename, alignmentname, readlist):
    stream = subprocess.Popen(["LA4Awesome", filename, filename, alignmentname]+ map(str,readlist),
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment2(stream.stdout) # generator
    return alignments
    
    
def get_alignments_mapping(filename, ref, alignmentname, readlist):
    stream = subprocess.Popen(["LA4Awesome", filename, ref, alignmentname]+ map(str,readlist)+ ['-F'],
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment2(stream.stdout) # generator
    return alignments
    
def get_alignments_mapping2(ref, filename, alignmentname):
    print ref,filename,alignmentname
    stream = subprocess.Popen(["LA4Awesome", ref, filename, alignmentname],
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment2(stream.stdout) # generator
    return alignments


    
def get_alignments_mapping3(ref, filename, alignmentname, contig_no):
    print ref,filename,alignmentname
    stream = subprocess.Popen(["LA4Awesome", ref, filename, alignmentname, contig_no],
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment2(stream.stdout) # generator
    return alignments

def get_all_reads(filename):
    stream = subprocess.Popen(["DBshow", filename],
                                      stdout=subprocess.PIPE,bufsize=1)
    reads = parse_read(stream.stdout) # generator
    return reads

def get_all_alignments(filename):
    stream = subprocess.Popen(["LAshow", filename, filename ],
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment(stream.stdout) # generator
    return alignments

def get_all_alignments2(filename, alignmentname):
    stream = subprocess.Popen(["LA4Awesome", filename, filename, alignmentname ],
                                      stdout=subprocess.PIPE,bufsize=1)
    alignments = parse_alignment2(stream.stdout) # generator
    return alignments

def get_all_reads_in_alignment_with_one(filename,read):
    this_read = get_reads(filename,[read])
    alignments = list(get_alignments(filename,[read]))
    readlist = map(lambda x:x[2],alignments)
    print readlist
    other_reads = get_reads(filename,readlist)
    
    return [list(this_read), list(other_reads), alignments] # note that this is not a generator


# test   
#for item in get_reads('G',[1]):
#    print item
    
#for item in get_alignments('G',[1]):
#    print item

#for item in get_alignments2('G','G.1.las',[1]):
#    print item

#for item in get_all_reads_in_alignment_with_one('G',1):
#    print item

#for item in get_reads('G', [1,2,3]):
#    print item




#!/usr/bin/env python

import sys, os
from pbcore.io import FastaIO

def run(reader, writer, lookupfile):
    with open (lookupfile,'w') as f:
        for i,record in enumerate(reader):
            seq_length = len(record.sequence)

            zmw = i+1
            old_header=record.header

            if seq_length < 30:
                new_header = 'Deleted'
                f.write(old_header+'\t'+new_header+'\n')
                continue
            #bounds = record.header.split('/')[-1]
            #start, end = [int(k) for k in bounds.split('_')]
            start = 0
            new_end = start + seq_length

            new_header = "m000_000/{zmw}/{start}_{end}".format(zmw=zmw, start=start, end=new_end)
            f.write(old_header+'\t'+new_header+'\n')

            writer.writeRecord(new_header, record.sequence)

def main(iname, ofile, lookupfile):
    reader = FastaIO.FastaReader(iname)
    writer = FastaIO.FastaWriter(ofile)
    run(reader, writer,lookupfile)

if __name__ == '__main__':
    iname, oname, lookupfile = sys.argv[1:4]
    ofile = open(oname, 'w')
    try:
        main(iname, ofile, lookupfile)
    except:
        # clean up (for make)
        ofile.close()
        os.unlink(oname)
        raise



#!/usr/bin/env python
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from ipywidgets.widgets import interact
import interface_utils as util
import sys

import os
os.environ['PATH'] += ':/data/pacbio_assembly/AwesomeAssembler/DALIGNER'
#print os.popen("export").read()

left = int(sys.argv[1])
right = int(sys.argv[2])


#rst = range(1,1399)
path = '/data/pacbio_assembly/AwesomeAssembler/data/ecoli/'
aln = []

bb = []
with open('ecoli.linear.edges') as f:
    for line in f:
        e = line.split(" ")[0]
        if e[-1] == '\'':
            e = e[:-1]

        bb.append(int(e))

print bb

bb = set(bb)


for i,item in enumerate(util.get_alignments_mapping2(path+'draft', path +'ecoli', path +'draft.ecoli.las')):
    if i%2000 == 0:
        print i, item

    if item[3] >= left and item[4] <= right:
        aln.append(item)








print 'number:',len(aln)
aln.sort(key = lambda x:x[2])

alns = []
current_b = aln[0][2]
aln_group = []

for item in aln:
    if current_b != item[2]:
        alns.append(aln_group)
        aln_group = []
        aln_group.append(item)
        current_b = item[2]
    else:
        aln_group.append(item)

num = len(alns)

print len(aln), len(alns)

alns.sort(key = lambda x:min([item[3] for item in x]))


plt.figure(figsize = (15,10))
plt.axes()
#plt.gca().axes.get_yaxis().set_visible(False)
#l = aln[0][5]
tip = (right-left)/5000
ed = (right-left)/2000
grid_size = 1.0
plt.xlim(left-2000,right+2000)
plt.ylim(-5,num*grid_size)

points = [[left,0], [right,0], [right+tip,grid_size/4], [right,grid_size/2], [left,grid_size/2]]
#rectangle = plt.Rectangle((0, 0), l, 5, fc='r',ec = 'none')
polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.6)
plt.gca().add_patch(polygon)

dotted_line = plt.Line2D((left, left), (0, num*grid_size ),ls='-.')
plt.gca().add_line(dotted_line)

dotted_line2 = plt.Line2D((right, right), (0, num*grid_size ),ls='-.')
plt.gca().add_line(dotted_line2)

for i,aln_group in enumerate(alns):
    for item in aln_group:
        abpos = item[3]
        aepos = item[4]
        bbpos = item[6]
        bepos = item[7]
        blen = item[8]
        strand = item[0]
        points_start = []
        points_end = []

        if strand == 'n':
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size], [aepos + tip, (i+1)*grid_size + grid_size/4], [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos + ed+ tip, (i+1)*grid_size + grid_size/4],  [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size]]
        else:
            points = [[abpos, (i+1)*grid_size], [aepos, (i+1)*grid_size], [aepos, (i+1)*grid_size+grid_size/2], [abpos, (i+1)*grid_size+grid_size/2], [abpos - tip, (i+1)*grid_size + grid_size/4]]
            if (bepos < blen):
                points_end = [[aepos, (i+1)*grid_size],  [aepos, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size+grid_size/2], [aepos+ed, (i+1)*grid_size]]
            if (bbpos > 0):
                points_start = [[abpos, (i+1)*grid_size],[abpos-tip, (i+1)*grid_size+grid_size/4], [abpos, (i+1)*grid_size+grid_size/2], [abpos-ed, (i+1)*grid_size+grid_size/2],[abpos-ed-tip, (i+1)*grid_size+grid_size/4], [abpos-ed, (i+1)*grid_size]]

        if item[2] in bb:
            polygon = plt.Polygon(points,fc = 'r', ec = 'none', alpha = 0.8)
        else:
            polygon = plt.Polygon(points,fc = 'b', ec = 'none', alpha = 0.6)

        polygon.set_url("http://shannon.stanford.edu:5000/aln" + str(item[2]+1) + ".pdf")
        plt.gca().add_patch(polygon)

        if points_end != []:
            polygon2 = plt.Polygon(points_end,fc = 'g', ec = 'none', alpha = 0.6)
            plt.gca().add_patch(polygon2)

        if points_start != []:
            polygon2 = plt.Polygon(points_start,fc = 'g', ec = 'none', alpha = 0.6)
            plt.gca().add_patch(polygon2)

plt.savefig('mapping/map.' + str(left) +'_'+ str(right)+ '.svg')



#!/usr/bin/env python

import sys
import os
import subprocess

bact_name = "ecoli"

if len(sys.argv) >= 2:
	bact_id = sys.argv[1]

st_point = 0
if len(sys.argv) >= 3:
	st_point = int(sys.argv[2])

base_path='/data/pacbio_assembly/pb_data/NCTC/'+bact_id+"/"


if len(sys.argv) >= 4:
	base_path = sys.argv[3]



fasta_names = [x for x in os.listdir(base_path) if x.endswith('.fasta')]
assert len(fasta_names)==1

fasta_name = fasta_names[0]
bact_name = fasta_name.split('.fasta')[0]

print bact_name


if st_point <= 1:
	subprocess.call("rm -f *.db",shell=True,cwd=base_path)
	fasta2DB_cmd = "fasta2DB "+bact_name+' '+base_path+fasta_name
	print fasta2DB_cmd
	subprocess.check_output(fasta2DB_cmd.split(),cwd=base_path)

if st_point <= 2:
	DBsplit_cmd = "DBsplit -x500 -s100 "+bact_name
	print DBsplit_cmd
	subprocess.check_output(DBsplit_cmd.split(),cwd=base_path)

if st_point <= 3:
	subprocess.call("rm -f *.las",shell=True,cwd=base_path)
	daligner_cmd = "HPCdaligner -t5 "+bact_name
        daligner_shell_cmd = "csh -v daligner_cmd.sh"
	print daligner_cmd
	p = subprocess.call(daligner_cmd.split(),stdout=open(base_path+'daligner_cmd.sh','w') , cwd=base_path)
        p2 = subprocess.check_output(daligner_shell_cmd.split(), cwd=base_path)
if st_point <= 4:
	remove_cmd = "rm "+base_path+bact_name+".*."+bact_name+".*"
	print remove_cmd
	os.system(remove_cmd)

if st_point <= 5:
	LAmerge_cmd = "LAmerge "+bact_name+".las "+bact_name+".*.las"
	print LAmerge_cmd
	subprocess.check_output(LAmerge_cmd,cwd=base_path,shell=True)

if st_point <= 6:
	remove_cmd2 = "rm "+base_path+bact_name+".*.las"
	os.system(remove_cmd2)

if st_point <= 7:
	os.system("mkdir -p "+base_path+"log")

if st_point <= 8:
	DASqv_cmd = "DASqv -c100 "+bact_name+" "+bact_name+".las"
	subprocess.check_output(DASqv_cmd.split(),cwd=base_path)

if st_point <= 9:
	Reads_filter_cmd = "Reads_filter --db "+bact_name+" --las "+bact_name+".las -x "+bact_name+" --config ~/AwesomeAssembler/utils/nominal.ini"
	print Reads_filter_cmd
	subprocess.check_output(Reads_filter_cmd,cwd=base_path, shell=True)

if st_point <= 10:
	hinging_cmd = "hinging --db "+bact_name+" --las "+bact_name+".las -x "+bact_name+" --config ~/AwesomeAssembler/utils/nominal.ini -o "+bact_name
	print hinging_cmd
	subprocess.check_output(hinging_cmd, cwd=base_path, shell=True)

if st_point <= 11:
	pruning_cmd = "python ~/AwesomeAssembler/scripts/pruning_and_clipping.py "+bact_name+".edges.hinges "+bact_name+".hinge.list A"
	print pruning_cmd
	subprocess.check_output(pruning_cmd, cwd=base_path, shell=True)





#!/usr/bin/env python

import sys
import os
import subprocess
from parse_read import *
import numpy as np
import networkx as nx
import itertools



filedir = sys.argv[1]
filename = sys.argv[2]
consensus_name = sys.argv[3]
in_graphml_name = filedir + '/' + filename +'_draft.graphml'
map_filename = filedir + '/draft_map.txt'


g = nx.read_graphml(in_graphml_name)


gfaname = filedir + '/' + filename +'_consensus.gfa'
cols = np.loadtxt(map_filename, dtype=str,usecols=(1,))
del_contigs = np.nonzero(cols == 'Deleted')[0]


# consensus_contigs = []
# i = 0
# try:
#     with open(consensus_name) as f:
#         for  line in f:
#             if line[0] != '>':
#                 consensus_contigs.append(line.strip())
#                 i += 1
#                 while i in set(del_contigs):
#                     consensus_contigs.append('')
#                     print len()
#                     i += 1
# except:
#     pass

del_contig_ptr = 0
cols = np.loadtxt(map_filename, dtype=str,usecols=(1,))
del_contigs = np.nonzero(cols == 'Deleted')[0]

consensus_contigs = []
i = 0
with open(consensus_name) as f:
    for  line in f:
        if line[0] != '>':
            while del_contig_ptr < len(del_contigs) :
                if len(consensus_contigs) == del_contigs[del_contig_ptr]:
                    consensus_contigs.append('')
                    del_contig_ptr += 1
                else:
                    break
            consensus_contigs.append(line.strip())
            i += 1


nodes_to_keep = [x for x in g.nodes() if consensus_contigs[g.node[x]['contig_id']] != '' ]
h = g.subgraph(nodes_to_keep)


# for  i, vert in enumerate(h.nodes()):
#     print i, vert
#     try:
#         print i,len(h.node[vert]['path']), len(h.node[vert]['segment']), len(consensus_contigs[i])
#     except:
#         print len(h.nodes()), len(consensus_contigs)
#         raise

print 'Number of contigs'
print len(consensus_contigs), len(h.nodes())
# print [len(x) for x in consensus_contigs]


with open(gfaname,'w') as f:
    f.write("H\tVN:Z:1.0\n")
    for j,vert in enumerate(h.nodes()):
        
        i = h.node[vert]['contig_id']
        # print j, i

        seg = consensus_contigs[i]
        print(len(seg))
        seg_line = "S\t"+vert+"\t"+seg + '\n'
        f.write(seg_line)
    for edge in h.edges():
        edge_line = "L\t"+edge[0]+"\t+\t"+edge[1]+"\t+\t0M\n"
        f.write(edge_line)

#last =  h.nodes()[-1]
#print h.node[last]
#path_last = h.node[last]['path']



#for i in range(len(path_last)-1):
#    read_a = path_last[i]
#    read_b = path_last[i+1]
#    print read_a, read_b, in_graph.edge[read_a][read_b]

# for i,node in enumerate(h.nodes()):
#      h.node[node]['path'] = ';'.join(h.node[node]['path'])
# nx.write_graphml(h,out_graphml_name)






import networkx as nx
import sys
import json

graphml_file = sys.argv[1]
groundtruth_file = sys.argv[2]
graphml_file_w_groundtruth = sys.argv[3]

g = nx.read_graphml(graphml_file)

print nx.info(g)

with open(groundtruth_file) as f:
    read_dict=json.load(f)

max_len=0
for read  in read_dict:
    for aln_info in read_dict[read]:
        try:
            max_len=max(max_len,len(str(aln_info[0])))
            max_len=max(max_len,len(str(aln_info[1])))
        except:
            print 
            raise

pow_mov=10**(max_len+1)

for node in g.nodes():
    #print node
    nodeid = node.split('_')[0]
    #print nodeid
    rev = int(node.split('_')[1])
    if rev==1:
        nodeid+="'"

    if nodeid in read_dict:
        g.node[node]['chr'] = read_dict[nodeid][0][2]
        g.node[node]['aln_end'] =  pow_mov*read_dict[nodeid][0][2]+max(read_dict[nodeid][0][0],read_dict[nodeid][0][1])
#         g.node[node]['aln_start'] = pow_mov*read_dict[nodeid][0][2]+min(read_dict[nodeid][0][0],read_dict[nodeid][0][1])
#         g.node[node]['repeat']=0
#         if len (read_dict[nodeid]) >1 :
#             g.node[node]['repeat']=1
#             chrom_maps=set([aln[3] for aln in read_dict[nodeid]])
#             if len (chrom_maps) >  1:
#                 g.node[node]['repeat']=10
    else:
        g.node[node]['chr'] = -1
        g.node[node]['aln_end'] =  -1
#         g.node[node]['aln_start'] = -1
#         g.node[node]['repeat']=-1

nx.write_graphml(g, graphml_file_w_groundtruth)



#!/usr/bin/env python

import networkx as nx
import random
import sys
from collections import Counter



# This script does a random condensation of the graph down to 2000 nodes

# python random_condensation.py ecoli.edges 2000

# It also keeps the ground truth on the graph through the condensation steps (if a json file is available)



def merge_path(g,in_node,node,out_node):
        
    g.add_edge(in_node,out_node,hinge_edge = -1,false_positive = 0)        
    g.remove_node(node)
    

def input1(flname):

    print "input1"

    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            if len(lines1) < 5:
                continue
            #print lines1
            g.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4], hinge_edge=int(lines1[5]))
            g.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))
    return g
            
def input2(flname):

    print "input2"

    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            g.add_edge(lines1[0], lines1[1])   
    return g



def input3(flname):

    print "input3"
    # g = nx.DiGraph()
    g = nx.read_graphml(flname)


def de_clip(filename, n_nodes, hinge_list,gt_file):

    n_iter = 5

    
    f=open(filename)
    line1=f.readline()
    print line1
    f.close()

    extension = filename.split('.')[-1]

    if extension == 'graphml':
        g=input3(filename)
    elif len(line1.split()) !=2:
        g=input1(filename)
    else:
        g=input2(filename)

    
    print nx.info(g)
    degree_sequence=sorted(g.degree().values(),reverse=True)
    print Counter(degree_sequence)
    
    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)
    
    try:
        import ujson
        mapping = ujson.load(open(gt_file))
        
        print 'getting mapping'
        mapped_nodes=0
        print str(len(mapping)) 
        print str(len(g.nodes()))
        for node in g.nodes():
            # print node
            node_base=node.split("_")[0]
            # print node_base

            #print node
            if mapping.has_key(node_base):
                g.node[node]['aln_start'] = min (mapping[node_base][0][0],mapping[node_base][0][1])
                g.node[node]['aln_end'] = max(mapping[node_base][0][1],mapping[node_base][0][0])
                g.node[node]['chr'] = mapping[node_base][0][2]
                mapped_nodes+=1
            else:
                # pass
                g.node[node]['aln_start'] = 0
                g.node[node]['aln_end'] = 0
                g.node[node]['aln_strand'] = 0


        for edge in g.edges_iter():
            in_node=edge[0]
            out_node=edge[1]
            # print  'akjdfakjhfakljh'
            if ((g.node[in_node]['aln_start'] < g.node[out_node]['aln_start'] and  
                g.node[out_node]['aln_start'] < g.node[in_node]['aln_end']) or 
                (g.node[in_node]['aln_start'] < g.node[out_node]['aln_end'] and
                g.node[out_node]['aln_end'] < g.node[in_node]['aln_end'])):
                g.edge[in_node][out_node]['false_positive']=0
            else:
                g.edge[in_node][out_node]['false_positive']=1

    except:
        raise
        # print "json "+filename.split('.')[0]+'.mapping.json'+" not found. exiting."
           
    print hinge_list

    print str(mapped_nodes)+" out of " +str(len(g.nodes()))+" nodes mapped."
    
    # for i in range(5):
    #     merge_simple_path(g)
    #     degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    #     print Counter(degree_sequence)

    in_hinges = set()
    out_hinges = set()
    num_iter=10000
    iter_done=0
    if hinge_list != None:
        print "Found hinge list."
        with open(hinge_list,'r') as f:
            for lines in f:
                lines1=lines.split()

                if lines1[2] == '1':
                  in_hinges.add(lines1[0]+'_0')
                  out_hinges.add(lines1[0]+'_1')
                elif lines1[2] == '-1':
                  in_hinges.add(lines1[0]+'_1')
                  out_hinges.add(lines1[0]+'_0')

        print str(len(in_hinges))+' hinges found.'

        for node in g.nodes():
            if node in in_hinges and node in out_hinges:
                g.node[node]['hinge']=100
            elif node in in_hinges:
                g.node[node]['hinge']=10
            elif node in out_hinges:
                g.node[node]['hinge']=-10
            else:
                g.node[node]['hinge']=0

        while len(g.nodes()) > n_nodes and iter_done < num_iter :
            node = g.nodes()[random.randrange(len(g.nodes()))]
            iter_done+=1
            # print iter_done
            if g.in_degree(node) == 1 and g.out_degree(node) == 1:

                base_node=node.split("_")[0]
                orintation = node.split("_")[1]
                # if orintation=='1':
                #     node2=base_node+'_0'
                # else:
                #     node2=base_node+'_1'

                # print node,node2

                in_node = g.in_edges(node)[0][0]
                out_node = g.out_edges(node)[0][1]

                if g.node[node]['hinge']==0 and g.node[in_node]['hinge']==0  and g.node[out_node]['hinge']==0:
                    if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                        if in_node != node and out_node != node and in_node != out_node:
                            bad_node=False
                            # print g.in_edges(node)
                            # print g.edge[g.in_edges(node)[0][0]][g.in_edges(node)[0][1]]
                            # print g.out_edges(node)
                            for in_edge in g.in_edges(node):
                                if g.edge[in_edge[0]][in_edge[1]]['false_positive']==1:
                                    bad_node=True
                            for out_edge in g.out_edges(node):
                                if g.edge[out_edge[0]][out_edge[1]]['false_positive']==1:
                                    bad_node=True 
                            if not bad_node:
                                #print in_node, node, out_node
                                merge_path(g,in_node,node,out_node)


                # print g.edge[edge1[0]][edge1[1]]['hinge_edge']

                for nd in g.nodes():
                    if len(nd.split("_"))==1:
                        print nd + " in trouble"
                # in_node = g.in_edges(node2)[0][0]
                # out_node = g.out_edges(node2)[0][1]
                # if g.node[node2]['hinge']==0 and g.node[in_node]['hinge']==0  and g.node[out_node]['hinge']==0:
                #     if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                #         if in_node != node2 and out_node != node2 and in_node != out_node:
                #             bad_node=False
                #             for in_edge in g.in_edges(node2):
                #                 if g.edge[in_edge]==1:
                #                     bad_node=True
                #             for out_edge in g.out_edges(node2):
                #                 if g.edge[out_edge]==1:
                #                     bad_node=True 
                #             if not bad_node:
                #                 #print in_node, node, out_node
                #                 merge_path(g,in_node,node2,out_node)


            # for nd in g.nodes():
            #     print nd

    else:
        while len(g.nodes()) > n_nodes:

            node = g.nodes()[random.randrange(len(g.nodes()))]



            if g.in_degree(node) == 1 and g.out_degree(node) == 1:

                # assert g.in_degree(node2) == 1 and g.out_degree(node2) == 1
                # edge_1 = g.out_edges(node)[0]
                # edge_2 = g.in_edges(node)[0]

                edge1 = g.out_edges(node)[0]
                edge2 = g.in_edges(node)[0]

                # print g.edge[edge1[0]][edge1[1]]['hinge_edge']

                if (g.edge[edge1[0]][edge1[1]]['hinge_edge'] == -1 and g.edge[edge2[0]][edge2[1]]['hinge_edge'] == -1):
                
                    in_node = g.in_edges(node)[0][0]
                    out_node = g.out_edges(node)[0][1]
                    if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                        if in_node != node and out_node != node and in_node != out_node:
                            #print in_node, node, out_node
                            merge_path(g,in_node,node,out_node)







    
    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)

    
    nx.write_graphml(g, filename.split('.')[0]+'.sparse3.graphml')
    
    print nx.number_weakly_connected_components(g)
    print nx.number_strongly_connected_components(g)
  

if __name__ == "__main__":   
    filename = sys.argv[1]
    try :
        hinge_list=sys.argv[3]
        print "Found hinge list."
    except:
        hinge_list=None
        print "in except "+hinge_list

    de_clip(filename, int(sys.argv[2]),hinge_list, sys.argv[4])








#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter


# This script condenses the graph down, creates a gfa with for the condensed graph, and computes the contig N50

# python condense_graph_create_gfa_compute_n50.py ecoli.edges

# The conditions in lines 23 and 24 are meant to prevent nodes corresponding to different strands to be merged 
# (and should be commented out if this is not desired, or if a json is not available)


def merge_simple_path(g):
    for node in g.nodes():
        if g.in_degree(node) == 1 and g.out_degree(node) == 1:
            
            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    if g.node[in_node]['aln_strand']==g.node[node]['aln_strand'] or max(g.node[in_node]['aln_strand'],g.node[node]['aln_strand']) == 5:
                        if g.node[out_node]['aln_strand']==g.node[node]['aln_strand'] or max(g.node[out_node]['aln_strand'],g.node[node]['aln_strand']) == 5:
                    #print in_node, node, out_node
                            merge_path(g,in_node,node,out_node)

def merge_path(g,in_node,node,out_node):
    #ov1 = find_overlap(g.node[in_node]['bases'], g.node[node]['bases'])
    #ov2 = find_overlap(g.node[node]['bases'], g.node[out_node]['bases'])
    
    node_id = g.graph['aval']
    g.graph['aval'] += 1
    #length = g.node[node]['length'] + g.node[in_node]['length'] + g.node[out_node]['length'] - ov1 - ov2
    #cov = (g.node[in_node]['cov'] * g.node[in_node]['length'] + g.node[node]['cov'] * g.node[node]['length']  + \
    #g.node[out_node]['cov'] * g.node[out_node]['length'])/float(length)
    #bases = g.node[in_node]['bases'][:-ov1] + g.node[node]['bases'] + g.node[out_node]['bases'][ov2:]

    overlap1 = g.edge[in_node][node][0]['overlap']
    overlap2 = g.edge[node][out_node][0]['overlap']

    length0 = g.node[in_node]['length']
    length1 = g.node[node]['length']
    length2 = g.node[out_node]['length']


    if overlap1 > min(length0,length1):
        print "problem here:"
        print overlap1, length0, length1


    g.add_node(str(node_id),length = length0+length1+length2 - overlap1 - overlap2, aln_strand = g.node[node]['aln_strand'])
    #g.add_node(str(node_id)+'-', bases = reverse_comp_bases(bases), length = length, cov = cov)
    
    for cur_edge in g.in_edges(in_node):

        # print g.edge[cur_edge[0]][cur_edge[1]][0]['overlap']

        g.add_edge(cur_edge[0],str(node_id),overlap = g.edge[cur_edge[0]][cur_edge[1]][0]['overlap'])
    
    for cur_edge in g.out_edges(out_node):
        g.add_edge(str(node_id),cur_edge[1],overlap = g.edge[cur_edge[0]][cur_edge[1]][0]['overlap'])

    
        
    g.remove_node(in_node)
    g.remove_node(node)
    g.remove_node(out_node)
    
def comp_n50(contig_vec):
    if len(contig_vec) == 0:
        return 0
    sorted_lengths = sorted(contig_vec)

    total_length = sum(contig_vec)

    half_length = 0.5*total_length

    min_n50 = sorted_lengths[-1]
    max_n50 = 0

    for i in range(len(sorted_lengths)):
    #if len(sorted_lengths) % 2 == 0:
    #  sum_1 = sum(sorted_lengths[0:i])
    #  sum_2 = sum(sorted_lengths[i:])
    #else:
    #  sum_1 = sum(sorted_lengths[0:i+1])
    #  sum_2 = sum(sorted_lengths[i:])
        sum_1 = sum(sorted_lengths[0:i+1])
        sum_2 = sum(sorted_lengths[i:])
        if sum_1 >= half_length and sum_2 >= half_length:
            min_n50 = min(sorted_lengths[i],min_n50)
            max_n50 = max(sorted_lengths[i],max_n50)

  # print "Min N50: "+str(min_n50)
  # print "Max N50: "+str(max_n50)

    return 0.5*(min_n50+max_n50)



def de_clip(filename, n_iter):

    g = nx.MultiDiGraph()
    
    # count = 0

    with open(filename,'r') as f:
        for line in f.xreadlines():
            l = line.strip().split()
            #print l2
            g.add_edge(l[0],l[1],overlap=int(l[2])/2)
            # if count < 10:
            #     print l[0], l[1], l[2]
            #     count += 1

            node0start = int(l[7][1:])
            node0end = int(l[8][:-1])

            g.node[l[0]]['length'] = node0end - node0start

            node1start = int(l[9][1:])
            node1end = int(l[10][:-1])

            g.node[l[1]]['length'] = node1end - node1start

    
    print nx.info(g)

    try:
        import ujson
        mapping = ujson.load(open(filename.split('.')[0]+'.mapping.json'))
        
        # print mapping

        print 'get mapping'
        
        for node in g.nodes():
            #print node
            if mapping.has_key(node):

                # alnstart = int(mapping[node][0])
                # alnend = int(mapping[node][1])

                # g.node[node]['length'] = abs(alnend-alnstart)
                # print abs(alnend-alnstart)

                g.node[node]['aln_strand'] = mapping[node][3]

                # g.node[node]['aln_start'] = mapping[node][0]
                # g.node[node]['aln_end'] = mapping[node][1]
                # g.node[node]['aln_strand'] = mapping[node][2]
            else:
                # g.node[node]['length'] = 5000
                g.node[node]['aln_strand'] = 5
                # print "this happened"
                # g.node[node]['aln_start'] = 0
                # g.node[node]['aln_end'] = 0
                # g.node[node]['aln_strand'] = 0
                
    except:
        pass 



    degree_sequence=sorted(g.degree().values(),reverse=True)
    print Counter(degree_sequence)
    for i in range(n_iter):
        for node in g.nodes():
            if g.degree(node) < 2:
                g.remove_node(node)

        print nx.info(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)

    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)
    
    
    g.graph['aval'] = 1000000000
    
    for i in range(5):
        merge_simple_path(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
       
    
    nx.write_graphml(g, filename.split('.')[0]+'.graphml')
    
    print nx.number_weakly_connected_components(g)
    print nx.number_strongly_connected_components(g)


    # Next we create the gfa file


    outputfile = filename.split('.')[0]+'.gfa'
    with open(outputfile, 'w') as fout:

        for cur_node in g.nodes():

            node_length = g.node[cur_node]['length']
            node_str = 'A'*node_length
            node_str = node_str + '\n'

            fout.write("NODE "+str(cur_node)+' 0 0 0 0 0\n')
            fout.write(node_str)
            fout.write(node_str)
            # print "NODE "+str(node)

        for arc in g.edges():
            fout.write("ARC "+str(arc[0])+' '+str(arc[1])+' 0\n')



    # Compute N50

    contig_lengths = []

    for cur_node in g.nodes():
        contig_lengths.append(g.node[cur_node]['length'])

    print "N50 = "+str(comp_n50(contig_lengths))



    
    
filename = sys.argv[1]
de_clip(filename, 5)



import sys

chr_lengths={}

ground_truth=sys.argv[1]
graph_file=sys.argv[2]
out_file=sys.argv[2]+'.clipped'

with open(ground_truth) as f:
    for line in f:
        m = map(int, line.strip().split())
        chr_lengths.setdefault(m[1],0)
        chr_lengths[m[1]]= max(chr_lengths[m[1]], max(m[2],m[3]))

CHR_THR=20000

reads_to_kill=set()

with open(ground_truth) as f:
    for line in f:
        m = map(int, line.strip().split())
        read_left=min(m[2],m[3])
        read_right=max(m[2],m[3])
        read_chr=m[1]
        if read_left < CHR_THR:
            reads_to_kill.add(m[0])
        if read_right > chr_lengths[read_chr] - CHR_THR:
            reads_to_kill.add(m[0])

with open(graph_file) as f:
    with open(out_file, 'w') as g:
        for line in f:
            line1=line.split()
            if int(line1[0])in reads_to_kill or int(line1[1]) in reads_to_kill:
                continue
            g.write(line)



#!/usr/bin/env python

import sys

def parse_read(stream = sys.stdin):    
    sid = ''
    seq = ''
    with stream as f:
        for l in f:
            if l[0] == '>':
                if sid == '':
                    sid = l[1:].strip()
                else:
                    tsid = sid
                    tseq = seq
                    seq = ''
                    sid = l[1:].strip()
                    yield (tsid,tseq)
                    
            else:
                seq += l.strip()
            
        yield(sid,seq)
                
                
#for read in parse_read():
#    print read
# do whatever you want to do with the reads



#!/usr/bin/env python

import networkx as nx
import sys

graphml_file = sys.argv[1]
groundtruth_file = sys.argv[2]
graphml_file_w_groundtruth = sys.argv[3]
try:
    chromosome_to_consider= int(sys.argv[4])
except:
    chromosome_to_consider=None

g = nx.read_graphml(graphml_file)

print nx.info(g)

mapping_dict = {}

with open(groundtruth_file,'r') as f:
    for num, line in enumerate(f.readlines()):
        m = map(int, line.strip().split())
        # mapping_dict[num] = [min(m), max(m), int(m[0]>m[1])]
        mapping_dict[num] = [m[2],m[3],m[1]]
        
#print mapping_dict

max_len=0
for num in mapping_dict.keys():
    max_len=max(max_len,len(str(m[3])))


pow_mov=10**(max_len+1)
for node in g.nodes():
    #print node
    try:
        nodeid = int(node.split('_')[0])
        #print nodeid
        rev = int(node.split('_')[1])
        if chromosome_to_consider != None:
            g.node[node]['chromosome'] = 0
            if mapping_dict[nodeid][2]==chromosome_to_consider:
                g.node[node]['chromosome'] = mapping_dict[nodeid][2]+1
        else:
            g.node[node]['chromosome'] = mapping_dict[nodeid][2]+1
        
        if rev == 0:
            g.node[node]['aln_end'] =  mapping_dict[nodeid][2]*pow_mov+ mapping_dict[nodeid][1]
            g.node[node]['aln_start'] = mapping_dict[nodeid][2]*pow_mov + mapping_dict[nodeid][0]
            # g.node[node]['aln_strand'] = mapping_dict[nodeid][2]
        else:
            g.node[node]['aln_end'] = mapping_dict[nodeid][2]*pow_mov + mapping_dict[nodeid][1]
            g.node[node]['aln_start'] = mapping_dict[nodeid][2]*pow_mov+ mapping_dict[nodeid][0]
            # g.node[node]['aln_strand'] = 1-mapping_dict[nodeid][2]
            
    except:
        pass
        
nx.write_graphml(g, graphml_file_w_groundtruth)





#!/usr/bin/env python

import sys
import re


def parse_alignment(stream = sys.stdin):    
    with stream as f:
        for l in f:
            sub = re.sub('[\[\].x:<difstraep()]',' ',l.strip())    
            sub = re.sub(',','',sub)
            lst = sub.split()[:-1]
            if len(lst) == 9:
                yield [lst[2]] + map(int, lst[0:2] + lst[3:])
                
                
def parse_alignment2(stream = sys.stdin):    
    with stream as f:
        for l in f:
            
            sub = re.sub('[\[\].x:<difstraep()/]',' ',l.strip())    
            sub = re.sub(',','',sub)
            lst = sub.split()[:-1]
            if len(lst) == 11:
                yield [lst[2]] + map(int, lst[0:2] + lst[3:])
                



#!/usr/bin/env python

# In[1]:

import networkx as nx
import sys

# In[2]:
if len(sys.argv) >2:
    print "wrong usage.\n python Visualise_graph.py graph_edge_file [list_of_hinges]"

vertices=set()
with open (sys.argv[1]) as f:

    for lines in f:
        lines1=lines.split()
        #print lines1
        if len(lines1) < 5:
            continue
        #vertices.add(lines1[0])
        #vertices.add(str(lines1[1]))
        #vertices.add(str(lines1[0])+"_" + lines1[3])
        #vertices.add(str(lines1[1])+"_" + lines1[4])


# In[3]:

len(vertices)


# In[4]:

G = nx.DiGraph()
for vertex in vertices:
    G.add_node(vertex)


# In[5]:

with open (sys.argv[1]) as f:
    for lines in f:
        lines1=lines.split()
        print lines1
        if len(lines1) < 5:
            continue
        #print lines1
        G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4], hinge_edge=int(lines1[5]))
        G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))

try:        
    in_hinges = set()
    out_hinges = set()
    with open (sys.argv[2]) as f:

	for lines in f:
	    lines1=lines.split()

	    if lines1[2] == '1':
		in_hinges.add(lines1[0]+'_0')
		out_hinges.add(lines1[0]+'_1')
	    elif lines1[2] == '-1':
		in_hinges.add(lines1[0]+'_1')
		out_hinges.add(lines1[0]+'_0')  
            
    for node in G.nodes():
	if node in in_hinges and node in out_hinges:
	    G.node[node]['hinge']=100
	elif node in in_hinges:
	    G.node[node]['hinge']=10
	elif node in out_hinges:
	    G.node[node]['hinge']=-10
	else:
	    G.node[node]['hinge']=0
except:
    pass 	    
        
nx.write_graphml(G, './out.graphml')

# In[ ]:






#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter
import json

LENGTH_THRESHOLD=10 #Connected components with less than LENGTH_THRESHOLD reads are thrown away


def merge_simple_path(g):
    for node in g.nodes():
        #print g.in_degree(node), g.out_degree(node)
        if g.in_degree(node) == 1 and g.out_degree(node) == 1:
            
            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    merge_path(g,in_node,node,out_node)
                    
                              
def merge_two_nodes(g):
    for node in g.nodes():
        if g.in_degree(node) == 1 and g.out_degree(node) == 0:
            in_node = g.in_edges(node)[0][0]
            if g.out_degree(in_node) == 1:
                if in_node != node:
                    node_id = g.graph['aval']
                    g.graph['aval'] += 1
                    g.add_node(str(node_id), 
                        count = g.node[in_node]['count'] + g.node[node]['count'],
                        read = g.node[in_node]['read'] + ':' + g.node[node]['read'],
                        #aln_chr = g.node[node]['aln_chr']
                        )
                    g.remove_node(in_node)
                    g.remove_node(node)
            

def merge_path(g,in_node,node,out_node):
    #ov1 = find_overlap(g.node[in_node]['bases'], g.node[node]['bases'])
    #ov2 = find_overlap(g.node[node]['bases'], g.node[out_node]['bases'])
    
    node_id = g.graph['aval']
    g.graph['aval'] += 1
    #length = g.node[node]['length'] + g.node[in_node]['length'] + g.node[out_node]['length'] - ov1 - ov2
    #cov = (g.node[in_node]['cov'] * g.node[in_node]['length'] + g.node[node]['cov'] * g.node[node]['length']  + \
    #g.node[out_node]['cov'] * g.node[out_node]['length'])/float(length)
    #bases = g.node[in_node]['bases'][:-ov1] + g.node[node]['bases'] + g.node[out_node]['bases'][ov2:]
    
    g.add_node(str(node_id), 
        count = g.node[in_node]['count'] + g.node[node]['count'] + g.node[out_node]['count'],
        read = g.node[in_node]['read'] + ':' +  g.node[node]['read'] + ':' +g.node[out_node]['read'],
        #aln_chr = g.node[node]['aln_chr']
    )
    #g.add_node(str(node_id)+'-', bases = reverse_comp_bases(bases), length = length, cov = cov)
    #print g.node[str(node_id)]['chr']

    for edge in g.in_edges(in_node):
        g.add_edge(edge[0],str(node_id))
    
    for edge in g.out_edges(out_node):
        g.add_edge(str(node_id),edge[1])
    
        
    g.remove_node(in_node)
    g.remove_node(node)
    g.remove_node(out_node)
    
def input1(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            if len(lines1) < 5:
                continue
            #print lines1
            g.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4], hinge_edge=int(lines1[5]))
            g.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))
    return g
            
def input2(flname):
    g = nx.DiGraph()
    with open (flname) as f:
        for lines in f:
            lines1=lines.split()
            #print lines1
            g.add_edge(lines1[0], lines1[1])   
    return g

def run(filename, gt_file, n_iter):
    
    
    f=open(filename)
    line1=f.readline()
    print line1
    f.close()
    if len(line1.split()) !=2:
	   g=input1(filename)
    else:
	   g=input2(filename)
    
    print str(len(g.nodes())) + " vertices in graph to begin with."

    connected_components=[x for x in nx.weakly_connected_components(g)]
    for component in connected_components:
        if len(component) < 10:
            g.remove_nodes_from(component)

    print str(len(g.nodes())) + " vertices in graph after removing components of at most "+str(LENGTH_THRESHOLD)+ " nodes."

    read_to_chr_map={}

    if gt_file.split('.')[-1]=='json':
        with open(gt_file,'r') as f:
            tmp_map=json.load(f)
        for read in tmp_map:
            readid=int(read.strip("'"))
            read_to_chr_map[readid] = int(tmp_map[read][0][2])
    else:
        with open(gt_file,'r') as f:
            for num, line in enumerate(f.readlines()):
                m = map(int, line.strip().split())
                read_to_chr_map[m[0]]=m[1]   
    
    nodes_seen=set([x.split("_")[0] for x in g.nodes()])

    for node in nodes_seen:
        read_to_chr_map.setdefault(int(node),-1)

    #print nx.info(g)
    print "Num reads read : "+str(len(read_to_chr_map))
    
    for node in g.nodes():
        nodeid=int(node.split('_')[0])

        g.node[node]['count'] = 1
        g.node[node]['read'] = node
        #print str(nodeid), node,g.node[node]['chr']
        
        
    degree_sequence=sorted(g.degree().values(),reverse=True)
    print Counter(degree_sequence)
    for i in range(n_iter):
        for node in g.nodes():
            if g.in_degree(node) == 0:
                g.remove_node(node)
    
        print nx.info(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    degree_sequence=sorted(nx.degree(g).values(),reverse=True)
    print Counter(degree_sequence)
    
    
    g.graph['aval'] = 1000000000
    
    for i in range(5):
        merge_simple_path(g)
        degree_sequence=sorted(nx.degree(g).values(),reverse=True)
        print Counter(degree_sequence)
    
    h=nx.DiGraph()
    h.add_nodes_from(g)
    h.add_edges_from(g.edges())
    for node in g.nodes():
        reads_in_node=[int(x.split('_')[0]) for x in g.node[node]['read'].split(':')]
        try:
            chr_in_node=map(lambda x: read_to_chr_map[x], reads_in_node)
        except:
            print reads_in_node,g.node[node]['read']
            return
        chr_in_node_set=set(chr_in_node)
        if len(chr_in_node_set) ==1:
            h.node[node]['chr']=chr_in_node[0]
        else:
            h.node[node]['chr']=':'.join(map(str,chr_in_node))

        h.node[node]['count']=g.node[node]['count']
        try:
            h.node[node]['read']=g.node[node]['read']
        except:
            pass

    
    nx.write_graphml(h, filename.split('.')[0]+'_condensed_annotated.graphml')
    
    print nx.number_weakly_connected_components(h)
    print nx.number_strongly_connected_components(h)
    
#

filename = sys.argv[1]
gt_file=sys.argv[2]
run(filename, gt_file,5)
